/****************************************************************************/
/* bit_io.c  (c) Copyright Black Ice Software Inc.  1989 - 1991             */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/*****************************************************************************
 Module
			c:\test\bitmap\bit_io.c

 Revision
			$Revision:   1.0  $

 Description
			functions to LOAD and SAVE WINDOWS bitmaps

 Notes
			

 
   $Log:   E:/c/ujtdemo/bit_io.c_v  $
 * 
 *    Rev 1.0   24 Jan 1991 10:54:20   Hrasko_Gabor
 * Initial revision.
 * 
 *    Rev 1.2   27 Aug 1990 14:30:08   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.1   20 Aug 1990 10:05:52   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.0   28 Jul 1990 20:16:20   Lorincz_Gabor
 * Initial revision.
 * 
 *    Rev 1.2   28 Jul 1990 12:42:20   Lorincz_Gabor
 * an error in lwrite() and in lread() was corrected,
 *     this error originated from original source of SDK!!!!!!
 * 
 *    Rev 1.1   28 Jul 1990 11:51:14   Lorincz_Gabor
 * saving and loading bitmap is OK if bitmap is smaller than 64K
 * when loading and saving, bitmap is dublicated into DIB!!!
 * when using in an independent application, be carefull with hPalette
 *      it must be updated!!!
 * 
 *    Rev 1.0   28 Jul 1990 10:57:36   Lorincz_Gabor
 * Initial revision.
*****************************************************************************/

//------------------------------- Include Files -------------------------------
#include <stdio.h>
#include <io.h>
#include <limits.h>
#include <windows.h>
//----------------------------- External Functions ----------------------------

//----------------------------- External Variables ----------------------------

//------------------------------- Module Defines ------------------------------

/* macro to determine if resource is a DIB */
#define ISDIB(bft) ((bft) == BFT_BITMAP)

/* Macro to determine to round off the given value to the closest byte */
#define WIDTHBYTES(i)	((i+31)/32*4)

/* Macros to display/remove hourglass cursor for lengthy operations */
#define StartWait() hcurSave = SetCursor(LoadCursor(NULL,IDC_WAIT))
#define EndWait()   SetCursor(hcurSave)




/* Header signatutes for various resources */
#define BFT_ICON   0x4349   /* 'IC' */
#define BFT_BITMAP 0x4d42   /* 'BM' */
#define BFT_CURSOR 0x5450   /* 'PT' */

// somee handy constants
#define PALVERSION      0x300
#define MAXPALETTE	256	  /* max. # supported palette entries */



//-------------------------------- Module Types -------------------------------

//------------------------------ Global Functions -----------------------------

//------------------------------ Global Variables -----------------------------
static HANDLE hcurSave ; // used by StartWait() and EndWait() macros
HANDLE hbiCurrent=NULL;
long dwOffset;
HPALETTE hpalCurrent=NULL;

BOOL	fPalColors  = FALSE;	      /* TRUE if the current DIB's color table   */
									      /* contains palette indexes not rgb values */

BOOL	bLegitDraw    = FALSE; /* We have a valid bitmap to draw	       */

BOOL	bMemoryDIB    = TRUE; /* Load Entire DIB into memory in CF_DIB format */

HANDLE	 hdibCurrent   = NULL;	       /* Handle to current memory DIB	       */

BOOL	bDIBToDevice  = FALSE; /* Use SetDIBitsToDevice() to BLT data.	       */

HBITMAP  hbmCurrent    = NULL;	       /* Handle to current memory BITMAP      */

//------------------------------ Module Functions -----------------------------
void     FreeDib(void);
HANDLE   LoadBitmapFromFile(LPSTR szFileName);
int      InitDIB(LPSTR szFileName);
HANDLE   ReadDibBitmapInfo (int fh);
BOOL     DibInfo (HANDLE hbi, LPBITMAPINFOHEADER lpbi);
void     ErrMsg (PSTR sz,...);
HANDLE   OpenDIB (LPSTR szFile);
HPALETTE CreateDibPalette (HANDLE hbi);
HPALETTE CreateBIPalette (LPBITMAPINFOHEADER  lpbi);
HBITMAP  BitmapFromDib (HANDLE	hdib, HPALETTE hpal);
WORD     DibNumColors (VOID FAR * pv);
WORD     PaletteSize (VOID FAR * pv);
DWORD    PASCAL lread (int fh, VOID FAR * pv, DWORD ul);
DWORD    PASCAL lwrite (int fh, VOID FAR *pv, DWORD ul);
BOOL     WriteDIB (LPSTR szFile, HANDLE hdib);
HANDLE   RealizeDibFormat (DWORD biStyle, DWORD biBits);
HANDLE   RenderFormat (int cf);
HANDLE   DibFromBitmap (HBITMAP hbm, DWORD biStyle, WORD biBits,HPALETTE  hpal);
HANDLE   CopyHandle (HANDLE h);
HPALETTE CopyPalette (HPALETTE hpal);

//------------------------------ Module Variables -----------------------------

//-------------------------------- Module Code --------------------------------
int SaveBitmap(LPSTR lpszFileName, HBITMAP hbmCurrent)
{
    BITMAPINFOHEADER bi;

		hbiCurrent=hdibCurrent = DibFromBitmap(hbmCurrent,
											BI_RGB,
											0, // compute from BITMAP structure
											NULL //hpalCurrent
											);
		// set some global variables in order to work with bitmaps receeived from clipboard
		fPalColors=TRUE;
		bLegitDraw=TRUE;
		bMemoryDIB=TRUE;
		// hPalCurrent?????

		DibInfo(hbiCurrent,&bi);

		bi.biCompression = BI_RGB;
//++++		bi.biBitCount=1; // this time it is set to 1!!!!!!!
									  // causes bitmap to be turned GRAY!!!

		/* Extract DIB specs. if the user did not press cancel */
		/* Realize a DIB in the specified format and obtain a
		* handle to it.
		*/
		hdibCurrent = RealizeDibFormat(bi.biCompression,bi.biBitCount);
		if (!hdibCurrent)
			{
				ErrMsg("Unable to save the specified file");
				return 0;
		   }

		/* Write the DIB */
		StartWait();
		if (!WriteDIB(lpszFileName,hdibCurrent))
			{
				ErrMsg("Unable to save the specified file");
			}
		EndWait();
		return(1);

}

HANDLE LoadBitmapFromFile(LPSTR szFileName)
{
 HANDLE hTemp;
	// loads DIB and converts into bitmap
	InitDIB( (LPSTR) szFileName);
	// now delete memory DIB
    if (hdibCurrent!=NULL)
		{
			GlobalFree(hdibCurrent);
			hdibCurrent=NULL;
		}
    if (hbiCurrent!= NULL)
		{
			GlobalFree(hbiCurrent);
			hbiCurrent=NULL;
		}
	//Delete the palette
    if (hpalCurrent)
	 {
	  DeleteObject(hpalCurrent);
	  fPalColors	= FALSE;
      bLegitDraw	= FALSE;
      hpalCurrent = NULL;
	 }
	hTemp = hbmCurrent;
    hbmCurrent	= NULL;

	 return(hTemp);
}



/*****************************************************************************
 Function
			InitDIB

 Parameters
			HWND
			szFileName: name of file

 Returns
			TRUE  - DIB loads ok				
			FALSE - otherwise

 Description
    Reads a DIB from a file, obtains a handle to it's          
 		 BITMAPINFO struct., sets up the palette and loads the DIB. 
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 09:32
*****************************************************************************/
int InitDIB(LPSTR szFileName)
{
    unsigned	       fh;
    LPBITMAPINFOHEADER lpbi;
    WORD FAR *	       pw;
    int 	       i;
    BITMAPINFOHEADER   bi;
    OFSTRUCT	       of;

    FreeDib();

    /* Open the file and get a handle to it's BITMAPINFO */

    fh = OpenFile (szFileName, (LPOFSTRUCT)&of, OF_READ);
    if (fh == -1)
	 {
			MessageBeep(100);
			return FALSE;
    }
    hbiCurrent = ReadDibBitmapInfo(fh);

    dwOffset = _llseek(fh, 0L, SEEK_CUR);
    _lclose (fh);

    if (hbiCurrent == NULL)
		{
			ErrMsg("%ls is not a Legitimate DIB File!", (LPSTR)szFileName);
			return FALSE;
	   }
    DibInfo(hbiCurrent,&bi);

    /* Set up the palette */
    
	hpalCurrent = CreateDibPalette(hbiCurrent);
    if (hpalCurrent == NULL)
		{
			ErrMsg("CreatePalette() Failed");
			return FALSE;
    	}

    /*	Convert the DIB color table to palette relative indexes, so
     *	SetDIBits() and SetDIBitsToDevice() can avoid color matching.
     *	We can do this because the palette we realize is identical
     *	to the color table of the bitmap, ie the indexes match 1 to 1
     *
     *	Now that the DIB color table is palette indexes not RGB values
     *	we must use DIB_PAL_COLORS as the wUsage parameter to SetDIBits()
     */
    lpbi = (VOID FAR *)GlobalLock(hbiCurrent);
    if (lpbi->biBitCount != 24)
		{
			fPalColors = TRUE;

			pw = (WORD FAR *)((LPSTR)lpbi + lpbi->biSize);

			for (i=0; i<(int)lpbi->biClrUsed; i++)
				{
		    		*pw++ = (WORD)i;
				}
    	}
    GlobalUnlock(hbiCurrent);
    bLegitDraw = TRUE;

    /*	If the input bitmap is not in RGB FORMAT the banding code will
     *	not work!  we need to load the DIB bits into memory.
     *	if memory DIB, load it all NOW!  This will avoid calling the
     *	banding code.
     */
    if (bMemoryDIB || bi.biCompression != BI_RGB)
		{
			if(!(hdibCurrent = OpenDIB(szFileName)))
            {      
			ErrMsg (" Not enough memory ! ");
			FreeDib();
			return FALSE;
			}
		}
    /*	If the RLE could not be loaded all at once, exit gracefully NOW,
     *	to avoid calling the banding code
     */
    if ((bi.biCompression != BI_RGB) && !hdibCurrent)
		{
			ErrMsg ("Could not load RLE!");
			FreeDib();
			return FALSE;
    	}

    if (hdibCurrent && !bDIBToDevice && bMemoryDIB)
		{
			hbmCurrent = BitmapFromDib(hdibCurrent,hpalCurrent);
			if (!hbmCurrent)
				{
	    			ErrMsg ("Could not create bitmap!");
	    			FreeDib();
	    			return FALSE;
				}
    	}


    return TRUE;
}
 																						
/*****************************************************************************
 Function
			FreeDIB

 Parameters
			None.

 Returns
			None.

 Description
			
  Frees all currently active bitmap, DIB and palette objects 
  and initializes their handles. 			    

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 09:35
*****************************************************************************/
void FreeDib(void)
{
    if (hpalCurrent)
	DeleteObject(hpalCurrent);

    if (hbmCurrent)
	DeleteObject(hbmCurrent);

    if (hdibCurrent)
	GlobalFree(hdibCurrent);

    if (hbiCurrent && hbiCurrent != hdibCurrent)
	GlobalFree(hbiCurrent);

    fPalColors	= FALSE;
    bLegitDraw	= FALSE;
    hpalCurrent = NULL;
    hdibCurrent = NULL;
    hbmCurrent	= NULL;
    hbiCurrent	= NULL;
}

/*****************************************************************************
 Function
			ReadDibBitmapInfo

 Parameters
			fh => DOS filehandler

 Returns
			A handle to the BITMAPINFO of the DIB in the file

 Description
  			Will read a file in DIB format and return a global HANDLE  
  			to it's BITMAPINFO.  This function will work with both     
  			"old" (BITMAPCOREHEADER) and "new" (BITMAPINFOHEADER)	    
  			bitmap formats, but will always return a "new" BITMAPINFO  
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 09:39
*****************************************************************************/
HANDLE ReadDibBitmapInfo (int fh)
{
    DWORD     off;
    HANDLE    hbi = NULL;
    int       size;
    int       i;
    WORD      nNumColors;

    RGBQUAD FAR       *pRgb;
    BITMAPINFOHEADER   bi;
    BITMAPCOREHEADER   bc;
    LPBITMAPINFOHEADER lpbi;
    BITMAPFILEHEADER   bf;
    DWORD	       dwWidth = 0;
    DWORD	       dwHeight = 0;
    WORD	       wPlanes, wBitCount;

    if (fh == -1)
        return NULL;

    /* Reset file pointer and read file header */
    off = _llseek(fh, 0L, SEEK_CUR);
    if (sizeof (bf) != _lread (fh, (LPSTR)&bf, sizeof (bf)))
        return FALSE;

    /* Do we have a RC HEADER? */
    if (!ISDIB (bf.bfType)) {
        bf.bfOffBits = 0L;
	_llseek (fh, off, SEEK_SET);
    }
    if (sizeof (bi) != _lread (fh, (LPSTR)&bi, sizeof(bi)))
        return FALSE;

    nNumColors = DibNumColors (&bi);

    /* Check the nature (BITMAPINFO or BITMAPCORE) of the info. block
     * and extract the field information accordingly. If a BITMAPCOREHEADER,
     * transfer it's field information to a BITMAPINFOHEADER-style block
     */
    switch (size = (int)bi.biSize){
	case sizeof (BITMAPINFOHEADER):
            break;

	case sizeof (BITMAPCOREHEADER):

	    bc = *(BITMAPCOREHEADER*)&bi;

	    dwWidth   = (DWORD)bc.bcWidth;
	    dwHeight  = (DWORD)bc.bcHeight;
	    wPlanes   = bc.bcPlanes;
	    wBitCount = bc.bcBitCount;

            bi.biSize               = sizeof(BITMAPINFOHEADER);
	    bi.biWidth		    = dwWidth;
	    bi.biHeight 	    = dwHeight;
	    bi.biPlanes 	    = wPlanes;
	    bi.biBitCount	    = wBitCount;

            bi.biCompression        = BI_RGB;
            bi.biSizeImage          = 0;
            bi.biXPelsPerMeter      = 0;
            bi.biYPelsPerMeter      = 0;
            bi.biClrUsed            = nNumColors;
            bi.biClrImportant       = nNumColors;

	    _llseek (fh, (LONG)sizeof (BITMAPCOREHEADER) - sizeof (BITMAPINFOHEADER), SEEK_CUR);
            break;

	default:
	    /* Not a DIB! */
	    return NULL;
    }

    /*	Fill in some default values if they are zero */
    if (bi.biSizeImage == 0){
	bi.biSizeImage = WIDTHBYTES ((DWORD)bi.biWidth * bi.biBitCount)
			 * bi.biHeight;
    }
    if (bi.biClrUsed == 0)
	bi.biClrUsed = DibNumColors(&bi);

    /* Allocate for the BITMAPINFO structure and the color table. */
    hbi = GlobalAlloc (GHND, (LONG)bi.biSize + nNumColors * sizeof(RGBQUAD));
    if (!hbi)
        return NULL;
    lpbi = (VOID FAR *)GlobalLock (hbi);
    *lpbi = bi;

    /* Get a pointer to the color table */
    pRgb = (RGBQUAD FAR *)((LPSTR)lpbi + bi.biSize);
    if (nNumColors){
	if (size == sizeof(BITMAPCOREHEADER)){
	    /* Convert a old color table (3 byte RGBTRIPLEs) to a new
	     * color table (4 byte RGBQUADs)
             */
	    _lread (fh, (LPSTR)pRgb, nNumColors * sizeof(RGBTRIPLE));

	    for (i = nNumColors - 1; i >= 0; i--){
                RGBQUAD rgb;

                rgb.rgbRed      = ((RGBTRIPLE FAR *)pRgb)[i].rgbtRed;
                rgb.rgbBlue     = ((RGBTRIPLE FAR *)pRgb)[i].rgbtBlue;
                rgb.rgbGreen    = ((RGBTRIPLE FAR *)pRgb)[i].rgbtGreen;
                rgb.rgbReserved = (BYTE)0;

                pRgb[i] = rgb;
            }
        }
	else
            _lread(fh,(LPSTR)pRgb,nNumColors * sizeof(RGBQUAD));
    }

    if (bf.bfOffBits != 0L)
        _llseek(fh,off + bf.bfOffBits,SEEK_SET);

    GlobalUnlock(hbi);
    return hbi;
}
//--------------------------------------------------------------------
void ErrMsg (PSTR sz,...)
{
    char ach[128];

    wvsprintf (ach, sz, (LPSTR)(&sz+1));   /* Format the string */
    MessageBox (NULL, ach, NULL, MB_OK|MB_ICONEXCLAMATION|MB_APPLMODAL);
}

/*****************************************************************************
 Function
			DibInfo

 Parameters
			

 Returns
			BOOL

 Description
			
 			Retrieves the DIB info associated with a CF_DIB
 			format memory block

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 09:58
*****************************************************************************/
BOOL DibInfo (HANDLE hbi, LPBITMAPINFOHEADER lpbi)
{
    if (hbi)
		{
			*lpbi = *(LPBITMAPINFOHEADER)GlobalLock (hbi);

			/* fill in the default fields */
			if (lpbi->biSize != sizeof (BITMAPCOREHEADER))
				{
            			if (lpbi->biSizeImage == 0L)
								{
									lpbi->biSizeImage =
		    							WIDTHBYTES(lpbi->biWidth*lpbi->biBitCount) * lpbi->biHeight;
								}
            			if (lpbi->biClrUsed == 0L)
								{
									lpbi->biClrUsed = DibNumColors (lpbi);
								}
        		}
			GlobalUnlock (hbi);
			return TRUE;
      }
    return FALSE;
}


/*****************************************************************************
 Function
			OpenDIB

 Parameters
			name of file

 Returns
			A handle to the DIB

 Description
 			Open a DIB file and create a MEMORY DIB, a memory handle
 			containing BITMAPINFO, palette data and the bits
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 10:08
*****************************************************************************/
HANDLE OpenDIB (LPSTR szFile)
{
    unsigned		fh;
    BITMAPINFOHEADER	bi;
    LPBITMAPINFOHEADER  lpbi;
    DWORD		dwLen = 0;
    DWORD		dwBits;
    HANDLE		hdib;
    HANDLE              h;
    OFSTRUCT		of;


    /* Open the file and read the DIB information */
    fh = OpenFile(szFile, &of, OF_READ);
    if (fh == -1)
	return NULL;

    hdib = ReadDibBitmapInfo(fh);
    if (!hdib)
	return NULL;
    DibInfo(hdib,&bi);

    /* Calculate the memory needed to hold the DIB */
    dwBits = bi.biSizeImage;
    dwLen  = bi.biSize + (DWORD)PaletteSize (&bi) + dwBits;

    /* Try to increase the size of the bitmap info. buffer to hold the DIB */
    h = GlobalReAlloc(hdib, dwLen, 0);
    if (!h){
	GlobalFree(hdib);
	hdib = NULL;
    }
    else
	hdib = h;

    /* Read in the bits */
    if (hdib){
        lpbi = (VOID FAR *)GlobalLock(hdib);
	lread(fh, (LPSTR)lpbi + (WORD)lpbi->biSize + PaletteSize(lpbi), dwBits);
	GlobalUnlock(hdib);
    }
    _lclose(fh);
    return hdib;
}

/*****************************************************************************
 Function
			CreateDibPalette

 Parameters
			None.

 Returns
			None.

 Description
 			Given a Global HANDLE to a BITMAPINFO Struct	
 			will create a GDI palette object from the color table
 			(BITMAPINFOHEADER format DIBs only)

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 10:13
*****************************************************************************/
HPALETTE CreateDibPalette (HANDLE hbi)
{
    HPALETTE hpal;

    if (!hbi)
		{
			return NULL;
		}
    hpal = CreateBIPalette((LPBITMAPINFOHEADER)GlobalLock(hbi));
    GlobalUnlock(hbi);
    return hpal;
}


/*****************************************************************************
 Function
			CreateBIPalette

 Parameters
			a pointer to BITMAPINFO

 Returns
			A handle to the palette

 Description
 			Given a Pointer to a BITMAPINFO struct will create a	
 			a GDI palette object from the color table
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 11:05
*****************************************************************************/
HPALETTE CreateBIPalette (LPBITMAPINFOHEADER  lpbi)
{
    LOGPALETTE          *pPal;
    HPALETTE            hpal = NULL;
    WORD                nNumColors;
    BYTE                red;
    BYTE                green;
    BYTE                blue;
    int                 i;
    RGBQUAD        FAR *pRgb;

    if (!lpbi)
	return NULL;

    if (lpbi->biSize != sizeof(BITMAPINFOHEADER))
	return NULL;

    /* Get a pointer to the color table and the number of colors in it */
    pRgb = (RGBQUAD FAR *)((LPSTR)lpbi + (WORD)lpbi->biSize);
    nNumColors = DibNumColors(lpbi);

    if (nNumColors){
	/* Allocate for the logical palette structure */
        pPal = (LOGPALETTE*)LocalAlloc(LPTR,sizeof(LOGPALETTE) + nNumColors * sizeof(PALETTEENTRY));
	if (!pPal)
	    return NULL;

        pPal->palNumEntries = nNumColors;
	pPal->palVersion    = PALVERSION;

	/* Fill in the palette entries from the DIB color table and
	 * create a logical color palette.
	 */
	for (i = 0; i < nNumColors; i++){
            pPal->palPalEntry[i].peRed   = pRgb[i].rgbRed;
            pPal->palPalEntry[i].peGreen = pRgb[i].rgbGreen;
            pPal->palPalEntry[i].peBlue  = pRgb[i].rgbBlue;
            pPal->palPalEntry[i].peFlags = (BYTE)0;
        }
        hpal = CreatePalette(pPal);
        LocalFree((HANDLE)pPal);
    }
    else if (lpbi->biBitCount == 24){
	/* A 24 bitcount DIB has no color table entries so, set the number of
	 * to the maximum value (256).
	 */
	nNumColors = MAXPALETTE;
        pPal = (LOGPALETTE*)LocalAlloc(LPTR,sizeof(LOGPALETTE) + nNumColors * sizeof(PALETTEENTRY));
        if (!pPal)
	    return NULL;

	pPal->palNumEntries = nNumColors;
	pPal->palVersion    = PALVERSION;

	red = green = blue = 0;

	/* Generate 256 (= 8*8*4) RGB combinations to fill the palette
	 * entries.
	 */
	for (i = 0; i < pPal->palNumEntries; i++){
            pPal->palPalEntry[i].peRed   = red;
            pPal->palPalEntry[i].peGreen = green;
            pPal->palPalEntry[i].peBlue  = blue;
            pPal->palPalEntry[i].peFlags = (BYTE)0;

	    if (!(red += 32))
		if (!(green += 32))
		    blue += 64;
        }
        hpal = CreatePalette(pPal);
        LocalFree((HANDLE)pPal);
    }
    return hpal;
}


/*****************************************************************************
 Function
			BitmapFromDIB

 Parameters
			None.

 Returns
			A handle to the DDB
 Description
 			Will create a DDB (Device Dependent Bitmap) given a global
 			handle to a memory block in CF_DIB format
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 10:29
*****************************************************************************/
HBITMAP BitmapFromDib (HANDLE	hdib, HPALETTE hpal)
{
    LPBITMAPINFOHEADER	lpbi;
    HPALETTE		hpalT;
    HDC 		hdc;
    HBITMAP		hbm;

    StartWait();

    if (!hdib)
		{
			return NULL;
		}

    lpbi = (VOID FAR *)GlobalLock(hdib);

    if (!lpbi)
		{
			return NULL;
		}

    hdc = GetDC(NULL);

    if (hpal){
        hpalT = SelectPalette(hdc,hpal,FALSE);
        RealizePalette(hdc);     // GDI Bug...????
    }

    hbm = CreateDIBitmap(hdc,
                (LPBITMAPINFOHEADER)lpbi,
                (LONG)CBM_INIT,
                (LPSTR)lpbi + lpbi->biSize + PaletteSize(lpbi),
                (LPBITMAPINFO)lpbi,
                DIB_RGB_COLORS );

    if (hpal)
		{
        SelectPalette(hdc,hpalT,FALSE);
		}

    ReleaseDC(NULL,hdc);
    GlobalUnlock(hdib);

    EndWait();

    return hbm;
}

/*****************************************************************************
 Function
			DibNumColors

 Parameters
			long pointer to bitmapinfoheader

 Returns
		  The number of colors in the DIB

 Description
 			Determines the number of colors in the DIB by looking at
 			the BitCount filed in the info block
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 10:50
*****************************************************************************/
WORD DibNumColors (VOID FAR * pv)
{
    int 		bits;
    LPBITMAPINFOHEADER	lpbi;
    LPBITMAPCOREHEADER	lpbc;

    lpbi = ((LPBITMAPINFOHEADER)pv);
    lpbc = ((LPBITMAPCOREHEADER)pv);

    /*	With the BITMAPINFO format headers, the size of the palette
     *	is in biClrUsed, whereas in the BITMAPCORE - style headers, it
     *	is dependent on the bits per pixel ( = 2 raised to the power of
     *	bits/pixel).
     */
    if (lpbi->biSize != sizeof(BITMAPCOREHEADER)){
        if (lpbi->biClrUsed != 0)
            return (WORD)lpbi->biClrUsed;
        bits = lpbi->biBitCount;
    }
    else
        bits = lpbc->bcBitCount;

    switch (bits){
	case 1:
		return 2;
	case 4:
		return 16;
	case 8:
		return 256;
	default:
		/* A 24 bitcount DIB has no color table */
		return 0;
    }
}

/*****************************************************************************
 Function
			PaletteSize

 Parameters
			pointer to header

 Returns
			Palette size in number of bytes

 Description
 Calculates the palette size in bytes. If the info. block
 is of the BITMAPCOREHEADER type, the number of colors is
 multiplied by 3 to give the palette size, otherwise the 
 number of colors is multiplied by 4.							
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 10:56
*****************************************************************************/
WORD PaletteSize (VOID FAR * pv)
{
    LPBITMAPINFOHEADER lpbi;
    WORD	       NumColors;

    lpbi      = (LPBITMAPINFOHEADER)pv;
    NumColors = DibNumColors(lpbi);

    if (lpbi->biSize == sizeof(BITMAPCOREHEADER))
        return NumColors * sizeof(RGBTRIPLE);
    else
        return NumColors * sizeof(RGBQUAD);
}


/*****************************************************************************
 Function
			WriteDIB

 Parameters
			filename
			handle of DIB

 Returns
			BOOL

 Description
				Write a global handle in CF_DIB format to a file			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/28/90 09:28
*****************************************************************************/
BOOL WriteDIB (LPSTR szFile, HANDLE hdib)
{
    BITMAPFILEHEADER	hdr;
    LPBITMAPINFOHEADER  lpbi;
    int                 fh;
    OFSTRUCT            of;

    if (!hdib)
	return FALSE;

    fh = OpenFile (szFile, &of, OF_CREATE|OF_READWRITE);
    if (fh == -1)
	return FALSE;

    lpbi = (VOID FAR *)GlobalLock (hdib);

    /* Fill in the fields of the file header */
    hdr.bfType		= BFT_BITMAP;
    hdr.bfSize		= GlobalSize (hdib) + sizeof (BITMAPFILEHEADER);
    hdr.bfReserved1     = 0;
    hdr.bfReserved2     = 0;
    hdr.bfOffBits       = (DWORD)sizeof(BITMAPFILEHEADER) + lpbi->biSize +
                          PaletteSize(lpbi);

    /* Write the file header */
    _lwrite (fh, (LPSTR)&hdr, sizeof (BITMAPFILEHEADER));

    /* Write the DIB header and the bits */
    lwrite (fh, (LPSTR)lpbi, GlobalSize (hdib));

    GlobalUnlock (hdib);
    _lclose (fh);
    return TRUE;
}


/*****************************************************************************
 Function
			RealizeDibFormat

 Parameters
			biStyle	  DIB format	  RGB or RLE
			biBits	  Bits per pixel  1,4,8,24

 Returns
			A handle to the created DIB

 Description
 			Realize the current DIB in the specifed format
 			This function is used to get a specific format of CF_DIB
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/28/90 09:31
*****************************************************************************/
HANDLE RealizeDibFormat (DWORD biStyle, DWORD biBits)
{
    BITMAPINFOHEADER bi;

    if (!bLegitDraw)
        return NULL;

    DibInfo (hbiCurrent, &bi);

    /*	Do we have the requested format already? */
    if (bi.biCompression == biStyle && bi.biBitCount == biBits){
        if (!hdibCurrent)
	    hdibCurrent = RenderFormat (CF_DIB);
    }
    else{
        if (!hbmCurrent)
	    hbmCurrent = RenderFormat (CF_BITMAP);

	if (hbmCurrent){
            if (hdibCurrent)
		GlobalFree (hdibCurrent);

	    hdibCurrent = DibFromBitmap (hbmCurrent, biStyle, biBits, hpalCurrent);
        }
    }

    return hdibCurrent;
}

/*****************************************************************************
 Function
			DibFromBitmap

 Parameters
			bitmap handle
			style info (BI_RGB ...)
			biBits info (if 0, bm.bmPlanes * bm.bmBitsPixel is used)
			palette handle (if NULL default is used)

 Returns
			A handle to the DIB

 Description
 			Will create a global memory block in DIB format that
 			represents the Device-dependent bitmap (DDB) passed in
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/28/90 10:10
*****************************************************************************/
HANDLE DibFromBitmap (HBITMAP hbm, DWORD biStyle, WORD biBits,HPALETTE  hpal)
{
    BITMAP               bm;
    BITMAPINFOHEADER     bi;
    BITMAPINFOHEADER FAR *lpbi;
    DWORD                dwLen;
    HANDLE               hdib;
    HANDLE               h;
    HDC                  hdc;

    if (hbm==NULL)
		{
			return NULL;
		}

    if (hpal == NULL)
		{
        hpal = GetStockObject(DEFAULT_PALETTE);
		}

    GetObject(hbm,sizeof(bm),(LPSTR)&bm);

    if (biBits == 0)
		{
			biBits =  bm.bmPlanes * bm.bmBitsPixel;
		}

    bi.biSize               = sizeof(BITMAPINFOHEADER);
    bi.biWidth              = bm.bmWidth;
    bi.biHeight             = bm.bmHeight;
    bi.biPlanes             = 1;
    bi.biBitCount           = biBits;
    bi.biCompression        = biStyle;
    bi.biSizeImage          = 0;
    bi.biXPelsPerMeter      = 0;
    bi.biYPelsPerMeter      = 0;
    bi.biClrUsed            = 0;
    bi.biClrImportant       = 0;

    dwLen  = bi.biSize + PaletteSize(&bi);

    hdc = GetDC(NULL);
    hpal = SelectPalette(hdc,hpal,FALSE);
	 RealizePalette(hdc);

    hdib = GlobalAlloc(GHND,dwLen);

    if (!hdib)
	 	{
			SelectPalette(hdc,hpal,FALSE);
			ReleaseDC(NULL,hdc);
			return NULL;
    	}

    lpbi = (VOID FAR *)GlobalLock(hdib);

    *lpbi = bi;

    /*	call GetDIBits with a NULL lpBits param, so it will calculate the
     *  biSizeImage field for us
     */
    GetDIBits(hdc, hbm, 0, (WORD)bi.biHeight,
      		  NULL, (LPBITMAPINFO)lpbi, DIB_RGB_COLORS);

    bi = *lpbi;
    GlobalUnlock(hdib);

    /* If the driver did not fill in the biSizeImage field, make one up */
    if (bi.biSizeImage == 0)
	 {
        bi.biSizeImage = WIDTHBYTES((DWORD)bm.bmWidth * biBits) * bm.bmHeight;

        if (biStyle != BI_RGB)
			{
            bi.biSizeImage = (bi.biSizeImage * 3) / 2;
			}
    }

    /*	realloc the buffer big enough to hold all the bits */
    dwLen = bi.biSize + PaletteSize(&bi) + bi.biSizeImage;
    if (h = GlobalReAlloc(hdib,dwLen,0))
		{
        hdib = h;
		}
    else
		{
        GlobalFree(hdib);
			hdib = NULL;

			SelectPalette(hdc,hpal,FALSE);
			ReleaseDC(NULL,hdc);
			return hdib;
    	}

    /*	call GetDIBits with a NON-NULL lpBits param, and actualy get the
     *  bits this time
     */
    lpbi = (VOID FAR *)GlobalLock(hdib);

    if (
				GetDIBits( hdc,
		   		hbm,
		   		0,
		   		(WORD)bi.biHeight,
		   		(LPSTR)lpbi + (WORD)lpbi->biSize + PaletteSize(lpbi),
		   		(LPBITMAPINFO)lpbi, DIB_RGB_COLORS)
				== 0
		 )
		{
	 		GlobalUnlock(hdib);
	 		hdib = NULL;
	 		SelectPalette(hdc,hpal,FALSE);
	 		ReleaseDC(NULL,hdc);
	 		return NULL;
    	}

    bi = *lpbi;
    GlobalUnlock(hdib);

    SelectPalette(hdc,hpal,FALSE);
    ReleaseDC(NULL,hdc);
    return hdib;
}



/*****************************************************************************
 Function
			RenderFormat

 Parameters
			None.

 Returns
			A handle to the DIB

 Description
			Renders the currently displayed DIB in CF_DIB or CF_BITMAP format
			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/28/90 09:48
*****************************************************************************/
HANDLE RenderFormat (int cf)
{
    HANDLE  h = NULL;
    HBITMAP hbm;

    if (!bLegitDraw)
        return NULL;

    switch (cf){
        case CF_BITMAP:
            	if (hbmCurrent)
					{
		    			h = BitmapFromDib (hdibCurrent, hpalCurrent);
					}
            	else
					{
               	h = NULL;
					}

            break;

        case CF_DIB:
                if (!hdibCurrent && hbmCurrent)
						{
		    				h = DibFromBitmap (hbmCurrent, BI_RGB, 0, hpalCurrent);
						}
                else if (hdibCurrent)
						{
		    				h = CopyHandle (hdibCurrent);
						}
                else
						{
                    h = NULL;
						}
            break;

        case CF_PALETTE:
            if (hpalCurrent)
					{
						h = CopyPalette (hpalCurrent);
					}
            break;
    }
    return h;
}

/*****************************************************************************
 Function
			CopyPalette

 Parameters
			
			source palette
 Returns
			A handle to the new palette

 Description
			Makes a copy of a GDI logical palette				

 Notes
			None.

 Author
			Lorincz_Gabor, 07/28/90 10:31
*****************************************************************************/
HPALETTE CopyPalette (HPALETTE hpal)
{
    PLOGPALETTE ppal;
    int         nNumEntries;

    if (!hpal)
	return NULL;

    GetObject(hpal,sizeof(int),(LPSTR)&nNumEntries);

    if (nNumEntries == 0)
        return NULL;

    ppal = (PLOGPALETTE)LocalAlloc(LPTR,sizeof(LOGPALETTE) +
                nNumEntries * sizeof(PALETTEENTRY));

    if (!ppal)
        return NULL;

    ppal->palVersion    = PALVERSION;
    ppal->palNumEntries = nNumEntries;

    GetPaletteEntries(hpal,0,nNumEntries,ppal->palPalEntry);

    hpal = CreatePalette(ppal);

    LocalFree((HANDLE)ppal);
    return hpal;
}

/*****************************************************************************
 Function
			CopyHandle

 Parameters
			template handle

 Returns
			copied handle

 Description

			Makes a copy of the given global memory block			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/28/90 10:26
*****************************************************************************/
HANDLE CopyHandle (HANDLE h)
{
    BYTE huge *lpCopy;
    BYTE huge *lp;
    HANDLE hCopy;
    DWORD  dwLen;

    dwLen = GlobalSize (h);
    if (hCopy = GlobalAlloc (GHND, dwLen))
		{
			lpCopy = (BYTE huge *)GlobalLock (hCopy);
			lp     = (BYTE huge *)GlobalLock (h);
			while (dwLen--)
				{
					*lpCopy++ = *lp++;
				}
			GlobalUnlock (hCopy);
			GlobalUnlock (h);
    	}
    return hCopy;
}


/*****************************************************************************
 Function
			lread

 Parameters
			filehandler
			buffer
			size

 Returns
 			0 - If read did not proceed correctly.
 			number of bytes read otherwise.			

 Description
			Reads data in steps of 32k till all the data has been read			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/27/90 11:01
*****************************************************************************/
DWORD PASCAL lread (int fh, VOID FAR * pv, DWORD ul)
{
    DWORD     ulT = ul;
    BYTE huge *hp = pv;

    while (ul > (DWORD)SHRT_MAX)
		{
			if (_lread(fh, (LPSTR)hp, SHRT_MAX) != SHRT_MAX)
				{
					return 0;
				}
			ul -= SHRT_MAX;
			hp += SHRT_MAX;
    	}
    if (_lread(fh, (LPSTR)hp, (WORD)ul) != (WORD)ul)
		{
			return 0;
		}
    return ulT;
}

/*****************************************************************************
 Function
			lwrite

 Parameters
			filehandler
			buffer to write
			size

 Returns
 			0 - If write did not proceed correctly.
 			number of bytes written otherwise.		 
			

 Description
			Writes data in steps of 32k till all the data is written			

 Notes
			None.

 Author
			Lorincz_Gabor, 07/28/90 10:35
*****************************************************************************/
DWORD PASCAL lwrite (int fh, VOID FAR *pv, DWORD ul)
{
    DWORD     ulT = ul;
    BYTE huge *hp = pv;

    while (ul > (DWORD)SHRT_MAX)
		{
			if (_lwrite(fh, (LPSTR)hp, SHRT_MAX) != SHRT_MAX)
				{
					return 0;
				}
			ul -= SHRT_MAX;
			hp += SHRT_MAX;
	   }
    if (_lwrite(fh, (LPSTR)hp, (WORD)ul) != (WORD)ul)
		{
			return 0;
		}
    return ulT;
}




//--------------------------------- Footnotes ---------------------------------

//-------------------------------- End of file --------------------------------



