/****************************************************************************/
/* clipboar.c  (c) Copyright Black Ice Software Inc.  1989 - 1991           */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/*****************************************************************************
 Module
			c:\test\blacky\clipboar.c

 Revision
			$Revision:   1.0  $

 Description
			clipboard related functions

 Notes
			

 
   $Log:   E:/c/ujtdemo/clipboar.c_v  $
 * 
 *    Rev 1.0   24 Jan 1991 10:54:38   Hrasko_Gabor
 * Initial revision.
 * 
 *    Rev 1.8   27 Aug 1990 14:26:04   Lorincz_Gabor
 * refinement on reveiving bitmaps larger than 64K
 * 
 *    Rev 1.7   20 Aug 1990 19:06:00   Lorincz_Gabor
 * PasteText() was improved
 *       bug, which cut right side of text was eliminated
 * 
 *    Rev 1.6   20 Aug 1990 10:00:12   Lorincz_Gabor
 * 
 * It can paste bitmaps larger than 64K
 * The need to test the size of the bitmap originated from literature
 * but suprisingly it is entirely needless moreover it is a wasted effort
 * 
 *    Rev 1.5   28 Jul 1990 20:15:48   Lorincz_Gabor
 * 
 *    Rev 1.4   25 Jul 1990 10:47:12   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.3   23 Jul 1990 12:31:10   Lorincz_Gabor
 * pasting metafile functionality was added and tested
 *    - if extent values in metafile header is set at least as aspect ratio
 *       function works well. Othervise there is no error message and error 
 *       condition, but received image may be infinitely small!!!
 *       I do not know what else to do when application which sends metafile
 *       to clipboard misbehaves, but do exist misbehaving apps.
 * an error in PasteBits() when closing clipboard was corrected
 * 
 *    Rev 1.2   14 Jul 1990  8:33:36   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.1   12 Jul 1990 16:11:20   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.0   11 Jul 1990  7:45:08   Lorincz_Gabor
 * Initial revision.
 * 
 *    Rev 1.0   06 Jul 1990  9:35:18   Lorincz_Gabor
 * Initial revision.
*****************************************************************************/

//------------------------------- Include Files -------------------------------

#include <stdlib.h>
#define NOMINMAX
#include <windows.h>
#include "data.h"
#include "fio.h"

//----------------------------- External Functions ----------------------------

//----------------------------- External Variables ----------------------------

extern GPAGETYPE gPage;

//------------------------------- Module Defines ------------------------------

//-------------------------------- Module Types -------------------------------

//------------------------------ Global Functions -----------------------------

//------------------------------ Global Variables -----------------------------

//------------------------------ Module Functions -----------------------------

HBITMAP FAR PasteBits( HWND hWnd );
HBITMAP FAR DuplicateBitmap( HBITMAP hbmOld );
BOOL FAR CopyToClipBoard(GPAGETYPE far *pPage);

//------------------------------ Module Variables -----------------------------

//-------------------------------- Module Code --------------------------------


/*****************************************************************************
 Function
			PasteBits

 Parameters
			hWnd

 Returns
			NULL if failed, otherwise handler of pasted bitmap

 Description
			

 Notes
			None.

 Author
			Lorincz_Gabor, 06/17/90 14:22
 HISTORY:
	Position of CloseClipboard() was changed 07/05/90 LG.
*****************************************************************************/
HBITMAP FAR PasteBits( HWND hWnd )
{
     HBITMAP hbmClip;                   /* handle to the clipboard's bitmap */
     HBITMAP hbmTemp=NULL;              /* temporary variable for new bitmap */

     /* ask for control of the clipboard */
     if (!OpenClipboard( hWnd ))
     {
          MessageBox( GetActiveWindow(), (LPSTR)"Could not open clipboard", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK);
          return( FALSE );
     }

     hbmClip = GetClipboardData( CF_BITMAP );     /* take bitmap */
     if (!hbmClip)                                /* did we get it? */
     {
		    CloseClipboard( );
          MessageBox( GetActiveWindow(), (LPSTR)"Could not receive bitmap from clipboard", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK);
          return( NULL );
     }

     /* make a copy to keep */
     hbmTemp = DuplicateBitmap( hbmClip );
          /* DuplicateBitmap gives its own error messages */
     CloseClipboard( );
     return( hbmTemp );
}

/*****************************************************************************
 Function
			DuplicateBitmap

 Parameters
			hBitmap => source bitmap

 Returns
			handle to duplicated bitmap

 Description
			Duplicates a bitmap

 Notes
			does not handle bitmaps lager than 64K

 Author
			Lorincz_Gabor, 06/17/90 14:32

 HISTORY:
			computing bitmapsize uses LONG!!!! - 07/05/90 LG.
*****************************************************************************/
HBITMAP FAR DuplicateBitmap( HBITMAP hbmOld )
{
     BITMAP bm;
     LONG nBytes;
     HANDLE hBits;
     LPSTR lpBits;
     HBITMAP hbmNew;

     /* get dimensions of old bitmap */
     GetObject( hbmOld, sizeof( BITMAP ), (LPSTR)&bm );

     /* check bitmap against size limit */
     nBytes = (long)bm.bmHeight * (long)bm.bmWidthBytes * (long)bm.bmPlanes;

     /* make a buffer for the image bits */
     if (!(hBits = GlobalAlloc( GHND, nBytes )))
     {
          MessageBox( GetActiveWindow(), (LPSTR)"Not enough memory to receive bitmap", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK );
          return( NULL );
     }

     /* lock that space and fill it with bits from hBitmap */
     lpBits = GlobalLock( hBits );
     if (GetBitmapBits( hbmOld, nBytes, lpBits ) != nBytes )
     {
          MessageBox( GetActiveWindow(), (LPSTR)"Can't copy bits from bitmap", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK );
          GlobalUnlock( hBits );
          GlobalFree( hBits );
          return( NULL );
     }

     bm.bmBits = lpBits ;
     hbmNew = CreateBitmapIndirect( &bm );        /* make duplicate bitmap */

     GlobalUnlock( hBits );                       /* release the buffer */
     GlobalFree( hBits );
     return( hbmNew );
}
/*****************************************************************************
 Function
			CopyToClipBoard

 Parameters
			None.

 Returns
			TRUE if succesful, othewise FALSE

 Description
			Creates os monochrome bitmap and draws selected objects into it,
			then sends bitmap to clipboard

 Notes
			

 Author
			Lorincz_Gabor, 06/18/90 11:35

			------------------------------------------------
			routine was changed, since only bitmaps must be copied
			------------------------------------------------


*****************************************************************************/
BOOL FAR CopyToClipBoard(GPAGETYPE far *pPage)
{
	HBITMAP hDuplicate;



	if (pPage->hCurrentBitmap==NULL)
		{
			return(FALSE);
		}

	if ((hDuplicate=DuplicateBitmap(pPage->hCurrentBitmap))==NULL)
		{
			MessageBox( GetActiveWindow(),
				(LPSTR)"Not enough memory to duplicate bitmap",
				(LPSTR)"TROUBLE",
				MB_ICONEXCLAMATION|MB_OK
			);
			return(FALSE);
		}

      if (!OpenClipboard( pPage->hWndDraw ))
      {
			MessageBox( GetActiveWindow(),
				(LPSTR)"Can't open clipboard",
				(LPSTR)"TROUBLE",
				MB_ICONEXCLAMATION|MB_OK
			);
			DeleteObject(hDuplicate);
			return(FALSE);
          
      }
      EmptyClipboard( );
      SetClipboardData( CF_BITMAP, hDuplicate );
      CloseClipboard( );
		

		// finish stuff
		return(TRUE);

}

/*****************************************************************************
 Function
			PasteText

 Parameters
			hWnd 	=> handler of caller window

 Returns
			handle of newly created bitmap which received the text

 Description
			Pastes CF_TEXT from clipboard 
			computes its size
			creates a bitmap large enough to receive the text
				(
					if bitmap can't be created, halves required sizes,
					this algorithm may make text unreadable, but programmer MUST
					allways try to satisfy request
				)

 Notes
			None.

 Author
			Lorincz_Gabor, 07/22/90 18:15
*****************************************************************************/

HBITMAP FAR PasteText( HWND hWnd )
{
    HANDLE hMemText;                   /* handle to the clipboards TEXT */
	  HDC hDC;
	  HBITMAP hBitMap;
	  HCURSOR hOldCur;

     /* ask for control of the clipboard */
     if (!OpenClipboard( hWnd ))
     {
          MessageBox( hWnd, (LPSTR)"Could not open clipboard", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK);
          return( FALSE );
     }

	   // Displays an hourglass-cursor and saves the original one.
	   hOldCur = SetCursor(LoadCursor(NULL,IDC_WAIT));

     hMemText = GetClipboardData( CF_TEXT );     /* take text */
     if (!hMemText)                              /* did we get it? */
     {
		   CloseClipboard( );
        MessageBox( hWnd, (LPSTR)"Could not receive bitmap from clipboard", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK);
        return( NULL );
     }
		hDC=GetDC(hWnd=GetActiveWindow());
     hBitMap = LoadASCIIIntoBitmap( hWnd, hMemText);
		ReleaseDC(hWnd, hDC);
		// Closes the clipboard
		CloseClipboard();
		// Restores original cursor from hourglass-cursor.
		SetCursor(hOldCur);
		return( hBitMap );

}




//--------------------------------- Footnotes ---------------------------------

//-------------------------------- End of file --------------------------------



