/****************************************************************************/
/* fio.c     (c) Copyright Black Ice Software Inc.  1989 - 1991             */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/*****************************************************************************
 Module
			c:\test\blacky\fio.c

 Revision
			$Revision:   1.1  $

 Description
			routines for saving and loading a file

 Notes
			

 
   $Log:   E:/c/ujtdemo/fio.c_v  $
 * 
 *    Rev 1.1   24 Jan 1991 10:54:50   Hrasko_Gabor
 * This is a reference version.
 * 
 *    Rev 1.0   09 Jan 1991 13:29:34   Hrasko_Gabor
 * Initial revision.
 * 
 *    Rev 1.7   27 Aug 1990 14:31:02   Lorincz_Gabor
 * loading *.msp filees was added
 * 
 *    Rev 1.6   20 Aug 1990 10:05:46   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.5   28 Jul 1990 20:16:12   Lorincz_Gabor
 * 
 *    Rev 1.4   28 Jul 1990 12:48:16   Lorincz_Gabor
 * save() and load() was extended to handle bitmaps
 *    SORRY these bitmap functions provide only WINDOWS.2XX format
 * 
 *    Rev 1.3   25 Jul 1990 10:47:54   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.2   14 Jul 1990  8:33:32   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.1   12 Jul 1990 16:11:18   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.0   11 Jul 1990  7:44:08   Lorincz_Gabor
 * Initial revision.
 * 
 *    Rev 1.0   06 Jul 1990  9:35:26   Lorincz_Gabor
 * Initial revision.
*****************************************************************************/

//------------------------------- Include Files -------------------------------

#include <stdio.h>  // sprintf()
#include <string.h> // strcat(),strcpy()
#include <io.h>
#include <fcntl.h>
#include <windows.h>
#include "data.h"
#include "fiodlg.h"
#include "pcx.h"
#include "formats.h"
#include "bit_io.h"
#include "clipboar.h"
#include "tdemo.h"

//----------------------------- External Functions ----------------------------

#ifdef TIFFPROG
	BOOL FAR PASCAL SaveBitmapInTiffFormat(LPSTR lpszFileName, HBITMAP hBm);
	BOOL FAR PASCAL SaveBitmapInCcittFormat(LPSTR lpszFileName, HBITMAP hBm, WORD wCompression);
	BOOL FAR PASCAL PutTiffIntoBitmap(LPSTR lpszFileName, int nImage);
	BOOL FAR PASCAL LoadCcittImage(LPSTR lpszFileName, WORD wCompression);
	BOOL FAR PASCAL LoadIBSImage(LPSTR lpszFileName);
#endif

//----------------------------- External Variables ----------------------------

extern GPAGETYPE gPage;
extern FORMATDESCRIPTOR FormatsArray[]; // in f_openna.c
extern int nDefaultFormat;

//------------------------------- Module Defines ------------------------------

//-------------------------------- Module Types -------------------------------

typedef char huge * HPSTR;

//------------------------------ Global Functions -----------------------------

BOOL    FAR PASCAL SaveFile(GPAGETYPE far *lpPage);
BOOL    FAR PASCAL LoadFile( LPSTR szFileName, int nFormatID);
void    SetBanner(GPAGETYPE *pPage);
HBITMAP LoadASCIIIntoBitmap(HWND hWnd,HANDLE hMem);

//------------------------------ Global Variables -----------------------------

//------------------------------ Module Functions -----------------------------

HWND LoadFunc(void);
void SaveFunc(GPAGETYPE *pPage);
BOOL LoadASCIIFileIntoBitmap(LPSTR szFileName);

//------------------------------ Module Variables -----------------------------

//-------------------------------- Module Code --------------------------------

/*****************************************************************************
 Function
			SetBanner

 Parameters
			HWND hWnd ==> window which caption title has to be changed

 Returns
			None.

 Description
			Changes window caption with currently edited file's name

 Notes
			None.

 Author
			Lorincz_Gabor, 06/21/90 08:51
*****************************************************************************/

void SetBanner(GPAGETYPE *pPage)
{

	if(pPage->hCurrentBitmap != NULL)
		{

		if(*pPage->szFileName != '\0')		// Filename defined.
			{
			char szBuffer[200];
			sprintf(szBuffer,"Image Name [%s]" ,pPage->szFileName);
			SetWindowText(pPage->hWndDraw,(LPSTR)szBuffer);
			}
		else				 				// Filename not defined.
			SetWindowText(pPage->hWndDraw,(LPSTR)"Image Name [Noname]");

		}

	else									// No bitmap at all.
		{
		SetWindowText(pPage->hWndDraw,(LPSTR)CAPTION_TEXT);
		*pPage->szFileName = '\0';
		}
		
}


/*****************************************************************************
 Function
			SaveFunc

 Parameters
			hWnd => main window handler, its caption will be changed

 Returns
			None.

 Description
			saves current drawing with current name

 Notes
			None.

 Author
			Lorincz_Gabor, 06/21/90 09:16
*****************************************************************************/

void SaveFunc(GPAGETYPE *pPage)
{
	char szDefaultName[150];
	char szRemember[150];
	OFSTRUCT LocalOf;
	int nFileHandler;


//	strcpy(szRemember,pPage->szFileName); // **** I moved this more below (HG).

	if (*pPage->szFileName == '\0')
	{
	// still no filename given
	lstrcpy((LPSTR)szDefaultName,(LPSTR)"*.*");
	if (!GetSaveAsFileName((GPAGETYPE far *)pPage, (LPSTR)szDefaultName))
		{
		return;
		}
	}

	strcpy(szRemember,pPage->szFileName); // ******** I moved that here (HG).

	// test whether file already exists
	if (OpenFile((LPSTR)szRemember,(LPOFSTRUCT)&LocalOf,OF_EXIST)!=-1)
		{
		// file already exists.
		// reopen and truncate file.
		nFileHandler=OpenFile((LPSTR)szRemember,(LPOFSTRUCT)&LocalOf,OF_READWRITE);
		if (nFileHandler==-1)
			{
			MessageBox(GetActiveWindow(),(LPSTR)"Unable to create file",
								(LPSTR)"ERROR",MB_ICONEXCLAMATION|MB_OK);
			return;
			}

		else
			{
			// first close the succesfully opened file.
			_lclose(nFileHandler);
			// set values into gPage
			OpenFile((LPSTR)pPage->szFileName,(LPOFSTRUCT)&pPage->of,OF_PARSE);
			if (SaveFile((GPAGETYPE far *)pPage))
				{
				SetBanner(pPage);
				pPage->bNeedSave=FALSE;
				}
			}
		}

	else
		{
		// file does not exist, create it
		nFileHandler=OpenFile((LPSTR)szRemember,(LPOFSTRUCT)&LocalOf,OF_CREATE|OF_READWRITE);
		if (nFileHandler==-1)
			{
			MessageBox(GetActiveWindow(),(LPSTR)"Unable to create file",
									(LPSTR)"ERROR",MB_ICONEXCLAMATION|MB_OK);
			return;
			}
		else
			{
			// first close the succesfully opened file
			_lclose(nFileHandler);
			// sets values into gPage and deletes the file.
			OpenFile((LPSTR)pPage->szFileName,(LPOFSTRUCT)&pPage->of,OF_DELETE);
			if (SaveFile((GPAGETYPE far *)pPage))
				{
				SetBanner(pPage);
				pPage->bNeedSave=FALSE;
				}
			}
		}

	return;
}


/*****************************************************************************
 Function
			LoadFunc

 Parameters
			hWnd => main window handler

 Returns
			None.

 Description
			processes "Open" menufunc

 Notes
			None.

 Author
			Lorincz_Gabor, 06/21/90 09:37
*****************************************************************************/
HWND LoadFunc()
{
	char szBuffer[150];
	int  nFormatID;
    HWND hChildWnd = NULL;
	int  i;

	// set default extension
	lstrcpy((LPSTR)szBuffer,(LPSTR)"*.*");
	for (i=0; *FormatsArray[i].szComboText; i++) {																	 

		if (nDefaultFormat==FormatsArray[i].nFormatID) {
		   lstrcpy((LPSTR)szBuffer,FormatsArray[i].szFileMatch);
      }
	}

	nFormatID = GetFileName((LPOFSTRUCT)&gPage.of,(LPSTR)szBuffer,(LPSTR)szBuffer);
	if (nFormatID != -1) {

      gPage.nFormatID = nDefaultFormat = nFormatID;
      gPage.bNeedSave = FALSE;
      
      // create child windows
      hChildWnd = CreateImageDisplay();

      // load requested file.
		if (LoadFile((LPSTR)szBuffer,nFormatID)) {

	     lstrcpy((LPSTR)gPage.szFileName,(LPSTR)szBuffer);

         if (hChildWnd) {
            // pass parameters to child
            SendMessage(hChildWnd, PM_IMGINIT, 0, (long)(LPSTR)&gPage);
         }
		}
      else {
         // kill child window
         SendMessage(hChildWnd, WM_CLOSE, 0, 0l);
      }
	}

    return hChildWnd;
}

/**********************************************************************************
  Function
		LoadFile

  Parameters
		None.

  Returns
		None.

  Description


  Notes
		None.

  Author
		Lorincz_Gabor
**********************************************************************************/

BOOL FAR PASCAL LoadFile( LPSTR szFileName, int nFormatID)
{
#ifdef PCXPROG
	HDC hDC;
	HWND hWnd;
	int nErrCode;
#endif

	switch (nFormatID)
		{

#ifdef TIFFPROG
			case TIFF_FORMAT:
				// The previous bitmap is erased in PutTiffIntoBitmap() only
				// if the process is confirmed in the displayed dialog box.
				return(PutTiffIntoBitmap(szFileName,0));

			case CCIT_31D_NOEOL_FORMAT:
			case CCIT_31D_EOL_FORMAT:
			case CCIT_32_FORMAT:
			case CCIT_4_FORMAT:
			case UNCOMPRESSED_FORMAT: 
				// The previous bitmap is erased in LoadCcittImage() only
				// if the process is confirmed in the displayed dialog box.
				return(LoadCcittImage(szFileName,nFormatID));
#endif

			case ASCII_FORMAT:
//				DeleteBitmap(TRUE);
				return (LoadASCIIFileIntoBitmap(szFileName));
				//return (LoadIBSImage(szFileName));
				break;



#ifdef PCXPROG
			case PCX_FORMAT:
//				DeleteBitmap(TRUE);
				hDC=GetDC(hWnd=GetActiveWindow());
				gPage.hCurrentBitmap=LoadPCXIntoBitmap(hDC,(LPSTR)szFileName,(int far*)&nErrCode, TRUE);
				ReleaseDC(hWnd,hDC);
				return ( (gPage.hCurrentBitmap==NULL) ? FALSE:TRUE  );
				break;
#endif

			case BITMAP_FORMAT:
//				DeleteBitmap(TRUE);
				gPage.hCurrentBitmap=LoadBitmapFromFile((LPSTR) szFileName);
				return ( (gPage.hCurrentBitmap==NULL) ? FALSE:TRUE  );
				break;

#ifndef TIFFPROG
			case CCIT_31D_NOEOL_FORMAT:
			case CCIT_31D_EOL_FORMAT:
			case CCIT_32_FORMAT:
			case CCIT_4_FORMAT:
			case UNCOMPRESSED_FORMAT: 
			case TIFF_FORMAT:
#endif
#ifndef PCXPROG
			case PCX_FORMAT:
#endif
				MessageBox(
					GetActiveWindow(),
					(LPSTR)"Format is not supported in this application."
					"\nContact Black Ice Software Inc. for other applications."
					"\nPhone (914) 277-7006    Fax (914) 276-8418",
					(LPSTR)"Attention",
					MB_ICONEXCLAMATION|MB_OK
					);
				break;

			default:
				MessageBox(
					GetActiveWindow(),
					(LPSTR)"Format is not supported",
					(LPSTR)"SORRY",
					MB_ICONEXCLAMATION|MB_OK
					);
				break;

		}

  	return (FALSE);
}




/**********************************************************************************
  Function
		SaveFile

  Parameters
		szFileName			Filename.
		nFormatID				Format ID.

  Returns
		None.

  Description
		Saves the bitmap in a given format.

  Notes
		TRUE on success, FALSE if failed.

  Author
		Lorincz_Gabor
**********************************************************************************/

BOOL FAR PASCAL SaveFile(GPAGETYPE far *lpPage)
{
	switch (lpPage->nFormatID)
		{

#ifdef TIFFPROG
			case TIFF_FORMAT:
				return(SaveBitmapInTiffFormat((LPSTR)lpPage->szFileName,lpPage->hCurrentBitmap));
				break;

			case CCIT_31D_NOEOL_FORMAT:
			case CCIT_31D_EOL_FORMAT:
			case CCIT_32_FORMAT:
			case CCIT_4_FORMAT:
			case UNCOMPRESSED_FORMAT: 
				return(SaveBitmapInCcittFormat((LPSTR)lpPage->szFileName,lpPage->hCurrentBitmap,lpPage->nFormatID));
				break;
#endif

#ifdef PCXPROG
			case PCX_FORMAT:

				#ifdef DEMO
					DEMO_MSG;
					return(FALSE);
				#endif
						 
				return(SaveBitmapInPCXFormat(lpPage->hCurrentBitmap,(LPSTR)lpPage->szFileName));
				break;
#endif
			case BITMAP_FORMAT:

				#ifdef DEMO
					DEMO_MSG;
					return(FALSE);
				#endif
			
				return((BOOL)SaveBitmap(lpPage->szFileName,lpPage->hCurrentBitmap));
				break;

#ifndef TIFFPROG
			case CCIT_31D_NOEOL_FORMAT:
			case CCIT_31D_EOL_FORMAT:
			case CCIT_32_FORMAT:
			case CCIT_4_FORMAT:
			case UNCOMPRESSED_FORMAT: 
			case TIFF_FORMAT:
#endif
#ifndef PCXPROG
			case PCX_FORMAT:
#endif
				MessageBox(
					GetActiveWindow(),
					(LPSTR)"Format is not supported in this application."
					"\nContact Black Ice Software Inc. for more information."
					"\nPhone (914) 277-7006    Fax (914) 276-8418",
					(LPSTR)"Attention",
					MB_ICONEXCLAMATION|MB_OK
					);
				break;

			default:
				MessageBox(
					GetActiveWindow(),
					(LPSTR)"Format is not supported.",
					(LPSTR)"SORRY",
					MB_ICONEXCLAMATION|MB_OK
					);
				return(FALSE);
				break;
		}
	return (TRUE);
}

/**********************************************************************************
  Function
		LoadASCIIFileToBitmap

  Parameters
		None.

  Returns
		None.

  Description


  Notes
		None.

  Author
		Vamosi_Janos,	01/29/91		Initial version.
**********************************************************************************/

BOOL LoadASCIIFileIntoBitmap(LPSTR szFileName)
{
	int nFileHandler;
	OFSTRUCT ofs;
	DWORD dwSize;
	HCURSOR hOldCur;
	HANDLE hRet;
	HWND hWnd;
	HDC hDC;
	VOID FAR *lpByte;

	nFileHandler=OpenFile(szFileName,(LPOFSTRUCT)&ofs,OF_READ);
	if (nFileHandler==-1)
		{
		return (FALSE);
		}
	// compute memory needed
	dwSize=filelength(nFileHandler);
	// allocate memory
	hRet=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,dwSize);
	if (hRet == NULL)
		{
		close(nFileHandler);
     MessageBox( NULL, (LPSTR)"Not enough memory !", 
			          (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK);
		return (FALSE);
		}

	// Displays an hourglass-cursor and saves the original one.
	hOldCur = SetCursor(LoadCursor(NULL,IDC_WAIT));

	// read text into memory
	lpByte= (VOID FAR *)GlobalLock(hRet);
	lread(nFileHandler,lpByte,dwSize); // capable to read more than 64K!!!!
	GlobalUnlock(hRet);
	close(nFileHandler);

	hDC=GetDC(hWnd=GetActiveWindow());
	gPage.hCurrentBitmap = LoadASCIIIntoBitmap(hWnd,hRet);
	ReleaseDC(hWnd, hDC);
	GlobalFree(hRet);

	// Restores original cursor from hourglass-cursor.
	SetCursor(hOldCur);

	return(gPage.hCurrentBitmap != NULL);
}

/**********************************************************************************
  Function
		LoadASCIIIntoBitmap

  Parameters
		hWnd: Handle of active window.
		hMem: Memory handle of the previously loaded text file.

  Returns
		A valid bitmap handle or NULL in case of any error.

  Description
		It loads the ASCII text located in the memory into a bitmap.

  Notes
		None.

  Author
		"Vamosi_Janos",	01/22/91		Initial version.
**********************************************************************************/

HBITMAP LoadASCIIIntoBitmap(HWND hWnd,HANDLE hMem)
{
 HPSTR hpRunner, hpszText;
 DWORD dwBlockSize;
 int nRows, nCols, nWidest;
 unsigned int i;
 TEXTMETRIC tm;
 int nWidth,nHeight;
 int nXExtent,nYExtent;
 int nCharHeight;
 HDC hDC,hdcBit;
 HBITMAP hbmTemp;
 RECT rRect;

 // Lock memory
	  hpRunner = hpszText = (HPSTR)GlobalLock(hMem);
	  if (hpszText==NULL)
		{
          MessageBox( hWnd, (LPSTR)"Could not lock the data area", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK);
          return( NULL );
		}
		
	  dwBlockSize = GlobalSize(hMem);	// Determines the size of memory block.
	  hpszText[dwBlockSize-1] = 0; 		// Zeroes the last byte of block.

		// compute number of rows, widest row
		nRows=1;
		nWidest=nCols=1;
		for (i=0; *hpRunner; i++) 
			{
				switch (*(hpRunner++))
					{
						case '\r': //CR
							nRows++;
							if (nCols>nWidest)
								{
									nWidest=nCols;
								}
							nCols=0;
							break;
						case '\t': //TAB
							nCols+=8;
							break;
						default:
							nCols++;
							break;
					}

			}
		// if last rows doesn t contain CR
		if (nCols>nWidest)
			{
				nWidest=nCols;
			}


		// compute appropriate bitmap dimensions
		hDC=GetDC(hWnd);
	  	hdcBit=CreateCompatibleDC(hDC);
	  	if (hdcBit==NULL)
			{
				ReleaseDC(hWnd,hDC);
				GlobalUnlock(hMem);
	          	MessageBox( hWnd, (LPSTR)"Unable to create DC", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK);
	          	return( NULL );
			}
     	ReleaseDC(hWnd,hDC);
		GetTextMetrics(hdcBit,(LPTEXTMETRIC)&tm);
		nXExtent = nWidth = (nWidest+5)*tm.tmAveCharWidth;
		nCharHeight = tm.tmAscent+tm.tmDescent+tm.tmExternalLeading;
		nYExtent = nHeight = (nRows+1) * nCharHeight;

		// ask API to compute surroinding rectangle
		SetRectEmpty( (LPRECT)&rRect);
		rRect.left  = 2 * tm.tmAveCharWidth;
		rRect.right = nXExtent;
		rRect.bottom= nYExtent;

		// It does not draw the text, only adjusts the size of the rectangle
	    DrawText(hdcBit,hpszText,-1,(LPRECT)&rRect, DT_CALCRECT|DT_EXPANDTABS|
                                            DT_EXTERNALLEADING|DT_LEFT);
		nWidth=nXExtent = rRect.right;
		nHeight=nYExtent = rRect.bottom;


		// try to create bitmap
		hbmTemp=NULL;
		do
			{
				// monochrome bitmap will do
				hbmTemp=CreateBitmap(
										nXExtent,	//width
										nYExtent,	//height
										1,				//planes
										1,				//bitcount
										(LPSTR)NULL // initialization
										);
				if (hbmTemp==NULL)
					{
						// decrement physical dimensions if bitmap would be too large
						nYExtent /=2;
					}
			}while ( (hbmTemp==NULL) && (nYExtent > nCharHeight) );

	  if (hbmTemp==NULL)
		{
		  DeleteDC(hdcBit);
		  GlobalUnlock(hMem);
          MessageBox( hWnd, (LPSTR)"Not enough memory !", (LPSTR)"ERROR", MB_ICONEXCLAMATION|MB_OK);
          return( NULL );
		}

	  // clear bitmap white
	  SelectObject(hdcBit,hbmTemp);
	  SelectObject(hdcBit,GetStockObject(WHITE_BRUSH));
	  PatBlt(hdcBit,0,0,nXExtent,nYExtent,PATCOPY);

	  // draw text into bitmap
	  SetMapMode(hdcBit,MM_ANISOTROPIC);
	  SetWindowExt(hdcBit,nWidth,nHeight);
	  SetViewportExt(hdcBit,nXExtent,nYExtent);

	  // If the height exceeds certain limlit (cc. 480)
	  // than it won't draw properly unless you use DT_NOCLIP !!??

	  DrawText(hdcBit,hpszText,-1,(LPRECT)&rRect, DT_NOCLIP|DT_EXPANDTABS|	
                                          DT_EXTERNALLEADING|DT_LEFT);

	  // finish job
	  DeleteDC(hdcBit);
 	  GlobalUnlock(hMem);
	  
      return( hbmTemp );

}  

//--------------------------------- Footnotes ---------------------------------

//-------------------------------- End of file --------------------------------

