/****************************************************************************/
/* f_openna.c  (c) Copyright Black Ice Software Inc.  1989 - 1991           */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/*****************************************************************************
 Module
			c:\test\ia\dialog\f_openna.c

 Revision
			$Revision:   1.0  $

 Description
			Gets a filename with specified path, name and extension

 Notes
			

 
   $Log:   E:/c/ujtdemo/f_openna.c_v  $
 * 
 *    Rev 1.0   24 Jan 1991 10:54:48   Hrasko_Gabor
 * Initial revision.
 * 
 *    Rev 1.8   27 Aug 1990 14:26:42   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.7   21 Aug 1990 12:52:36   Lorincz_Gabor
 * dialog fn. for default filaname was added
 * 
 *    Rev 1.6   20 Aug 1990 10:00:02   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.5   20 Aug 1990  9:56:48   Lorincz_Gabor
 * bug when changing format and dirlist did not change, was fixed
 * 
 *    Rev 1.4   28 Jul 1990 20:15:56   Lorincz_Gabor
 * 
 * 
 *    Rev 1.3   25 Jul 1990 10:47:58   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.2   14 Jul 1990  8:33:20   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.1   12 Jul 1990 16:11:10   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.0   11 Jul 1990  7:45:38   Lorincz_Gabor
 * Initial revision.
 * 
 *    Rev 1.0   06 Jul 1990  9:35:24   Lorincz_Gabor
 * Initial revision.
*****************************************************************************/

//------------------------------- Include Files -------------------------------

#include <stdio.h>
#include <io.h>
#include <direct.h> // getcwd()
#include <windows.h>
#include "data.h"
#include "ioname.h"
#include "saveas.h"
#include "formats.h"

//----------------------------- External Functions ----------------------------

//----------------------------- External Variables ----------------------------

extern int nDefaultFormat;

//------------------------------- Module Defines ------------------------------

//-------------------------------- Module Types -------------------------------

//------------------------------ Global Functions -----------------------------

//------------------------------ Global Variables -----------------------------

// constant texts for combobox and default file extensions
// null terminators of this array are important, since you may change its size
FORMATDESCRIPTOR FormatsArray[]=
{ 
		{"TIFF File Format ","*.tif",TIFF_FORMAT},
		{"Paintbrush (PCX)","*.pcx",PCX_FORMAT},
//	{"Windows metafile","*.wmf",METAFILE_FORMAT},
		{"Bitmap","*.bmp",BITMAP_FORMAT},
		{"CCITT Group 3 1D","*.",CCIT_31D_EOL_FORMAT},
		{"CCITT Group 3 1D NO EOL","*.",CCIT_31D_NOEOL_FORMAT},
		{"CCITT Group 3 2D ","*.",CCIT_32_FORMAT},
		{"CCITT Group 4","*.",CCIT_4_FORMAT},
		{"Raw image (mono) ","*.",UNCOMPRESSED_FORMAT},
//	{"Microsoft Paint Ve1","*.msp",MSPAINT1_FORMAT},
//	{"Microsoft Paint Ve2","*.msp",MSPAINT2_FORMAT},
//	{"Microsoft (BIFF) XLS","*.xls",XLS_FORMAT},
		{"ASCII Text","*.txt",ASCII_FORMAT},
//		{"IBS Image","*.img",ASCII_FORMAT},
		{"\0","\0",0}
};

int nFileFormat;

//------------------------------ Module Functions -----------------------------

HANDLE FAR PASCAL OpenDlg(HWND hDlg, unsigned message, WORD wParam, LONG lParam);
		// dialogBox to process OPEN request
HANDLE FAR PASCAL SaveAsDlg(HWND hDlg, unsigned message, WORD wParam, LONG lParam);
		// dialogbox to process SAVEAS request

BOOL FAR PASCAL GetFileName( LPOFSTRUCT lpof, LPSTR lpszName, LPSTR lpszExtension);
		// logical entry for OPEN request
BOOL FAR PASCAL GetSaveAsFileName(GPAGETYPE far *lpPage, LPSTR lpszExtension);

char FAR *lg_strchr( LPSTR szString, char CharToFind);
		// DLL equivalent of strchr()


void AddExt( LPSTR Name, LPSTR Ext);
void SeparateFile(LPSTR lpDestPath, LPSTR lpDestFileName, LPSTR lpSrcFileName);
void ChangeDefExt(LPSTR Ext, LPSTR Name);
void UpdateListBox(HWND hDlg);
void UpdateDirList(HWND hDlg);


//------------------------------ Module Variables -----------------------------

char szPath[150];
char szName[150];
char szExtension[150];

char szOpenName[128];
char szDefPath[128];

#ifdef PCXPROG
 char szDefSpec[13] = "*.pcx";
 char szDefExt[] = ".pcx";
#else
 char szDefSpec[13] = "*.tif";
 char szDefExt[] = ".tif";
#endif

char str[255];

//----------------------------- External Variables ----------------------------
extern int nDefaultFormat;

//-------------------------------- Module Code --------------------------------


/**********************************************************************************
  Function
		GetFileName

  Parameters
		None.

  Returns
		None.

  Description


  Notes
		None.

  Author
		Lorincz_Gabor
**********************************************************************************/

int FAR PASCAL GetFileName( LPOFSTRUCT lpOf, LPSTR lpszName, LPSTR lpszDefaultName)
{
	int nSuccess;
	FARPROC lpOpenDlg;
	OFSTRUCT LocalOf;
	int nFileHandler;
	HANDLE hIOInstance;
	int nReturn=-1;

	if (*lpszDefaultName)
		{
		// if user gave a default filename, use it
		lstrcpy( (LPSTR)str,lpszDefaultName);
	   SeparateFile( (LPSTR) str, (LPSTR) szDefSpec, lpszDefaultName);
		}

#ifdef PCXPROG
	nFileFormat=PCX_FORMAT; 
#else
	nFileFormat=TIFF_FORMAT; 
#endif

	if (nDefaultFormat>0)
		{
		nFileFormat=nDefaultFormat;
		}

	hIOInstance	= GetWindowWord( GetActiveWindow(),GWW_HINSTANCE);
	lpOpenDlg   = MakeProcInstance((FARPROC) OpenDlg, hIOInstance);
	nSuccess    = DialogBox(hIOInstance, "Open", GetActiveWindow(), lpOpenDlg);
	FreeProcInstance(lpOpenDlg);

	if (nSuccess == -1 || nSuccess == 0)
		{
		nReturn=-1;
		}
	else
		{
		if ((nFileHandler=OpenFile((LPSTR)szOpenName,(LPOFSTRUCT)&LocalOf,OF_EXIST))!=-1)
			{
			// file really exists, copy results
			lstrcpy(lpszName,(LPSTR)szOpenName);
			// fill ofstruct
			OpenFile((LPSTR)lpszName,(LPOFSTRUCT)lpOf,OF_EXIST);
			nReturn=nFileFormat; // gives fileformat ID back
			}
		}


	return(nReturn);
}


/**********************************************************************************
  Function
		GetSaveAsFileName

  Parameters
		None.

  Returns
		None.

  Description


  Notes
		None.

  Author
		Hrasko_Gabor,	01/30/91		ASCII is not allowed here.
		Lorincz_Gabor
**********************************************************************************/

BOOL FAR PASCAL GetSaveAsFileName(GPAGETYPE far *lpPage,LPSTR lpszTemplateName)
{
	int nSuccess;
	int bReturn=FALSE;
	FARPROC lpOpenDlg;
	OFSTRUCT LocalOf;
	int nFileHandler;
	HANDLE hIOInstance;
	int i;
	char szBuffer[10];

	// assemble all globals for dialog
#ifdef PCXPROG
	nFileFormat=PCX_FORMAT; 
#else
	nFileFormat=TIFF_FORMAT; 
#endif

	// The second condition is made to avoid ASCII format, which is not
	// supported when saving.
	if ((lpPage->nFormatID > 0) && (lpPage->nFormatID != ASCII_FORMAT))
		{
		nFileFormat = lpPage->nFormatID;
		}

	lstrcpy((LPSTR)szBuffer,(LPSTR)"*.*");
	for (i = 0 ; *FormatsArray[i].szComboText ; i++) 
		{
		if (nFileFormat == FormatsArray[i].nFormatID)
			{
			lstrcpy((LPSTR)szBuffer,(LPSTR)FormatsArray[i].szFileMatch);
			break;
			}
		}

	SeparateFile((LPSTR)str,(LPSTR)szDefSpec,(LPSTR)szBuffer);

	hIOInstance= GetWindowWord( GetActiveWindow(),GWW_HINSTANCE);
	lpOpenDlg = MakeProcInstance((FARPROC) SaveAsDlg, hIOInstance);
	nSuccess = DialogBox(hIOInstance, "Open", GetActiveWindow(), lpOpenDlg);
	FreeProcInstance(lpOpenDlg);

	if (nSuccess==-1 || nSuccess==0)
		{
		bReturn=FALSE;
		}
	else
		{
		if (OpenFile((LPSTR)szOpenName,(LPOFSTRUCT)&LocalOf,OF_EXIST)!=-1)
			{
			// file already exists, ask user
			if (MessageBox(GetActiveWindow(),(LPSTR)"File already exists",
					(LPSTR)"Overwrite it?",MB_ICONEXCLAMATION|MB_YESNO)==IDYES)
				{
				lstrcpy(lpPage->szFileName,(LPSTR)szOpenName);
				lpPage->nFormatID=nFileFormat;
				bReturn=TRUE;
				}
			else
				{
				bReturn=FALSE;
				}
			}
		else
			{
			// file does not exist, try to create it
			nFileHandler=OpenFile((LPSTR)szOpenName,(LPOFSTRUCT)&LocalOf,OF_CREATE|OF_READWRITE);
			if (nFileHandler==-1)
				{
				// could not create file
				bReturn=FALSE;
				}
			else
				{
				// first close succesfully opened file
				_lclose(nFileHandler);
				// copy results
				lstrcpy(lpPage->szFileName,(LPSTR)szOpenName);
				OpenFile((LPSTR)lpPage->szFileName,(LPOFSTRUCT)&lpPage->of,OF_EXIST);
				// delete file, not to influence save()
				OpenFile((LPSTR)lpPage->szFileName,(LPOFSTRUCT)&lpPage->of,OF_DELETE);
				lpPage->nFormatID=nFileFormat;
				bReturn=TRUE;
				}
			}
		}

	return(bReturn);
}




/**********************************************************************************
  Function
		OpenDlg

  Parameters
		None.

  Returns
		None.

  Description


  Notes
		None.

  Author
		Lorincz_Gabor
**********************************************************************************/

HANDLE FAR PASCAL OpenDlg(HWND hDlg, unsigned message, WORD wParam, LONG lParam)
{
	HANDLE hFile = 1;     /* Temp value for return */
	int i;
	HWND hWndCombo;
	int err;

	switch (message) 
		{
     case WM_COMMAND:
     	switch (wParam)
				{

				case IDC_DIRLIST:
					switch (HIWORD(lParam))
						{
						case LBN_SELCHANGE:
                 	/* If item is a directory name, append "*.*" */
                    if (DlgDirSelect(hDlg, str, IDC_DIRLIST))
								{
								if (str[1]==':')
									{
									// if user would change to an other drive
									lstrcat( (LPSTR)str,(LPSTR)"\\");
									lstrcat((LPSTR)str, (LPSTR)szDefSpec);
									}
								else
									{
		                     lstrcat((LPSTR)str, (LPSTR)szDefSpec);
									}
								}
							lstrcpy((LPSTR)szDefPath,(LPSTR)str);
							SetDlgItemText(hDlg, IDC_EDIT, str);
							SendDlgItemMessage(hDlg,IDC_EDIT,EM_SETSEL,NULL,MAKELONG(0, 0x7fff));

							break;

						case LBN_DBLCLK:
							goto openfile;
							break;
							}

						break;

					case IDC_LISTBOX:
	              switch (HIWORD(lParam))
							{
		
		               case LBN_SELCHANGE:
		               /* If item is a directory name, append "*.*" */
		               	if (DlgDirSelect(hDlg, str, IDC_LISTBOX))
									{
		                     lstrcat((LPSTR)str, (LPSTR)szDefSpec);
									}
		
								SetDlgItemText(hDlg, IDC_EDIT, str);
		                  SendDlgItemMessage(hDlg,IDC_EDIT,EM_SETSEL,NULL,MAKELONG(0, 0x7fff));
		
		                  break;
		
							case LBN_DBLCLK:
		                  goto openfile;
								break;
		
							}
	
                 return (TRUE);

					case IDC_FORMAT:
						switch (HIWORD(lParam))
							{
							case CBN_SELCHANGE:
							   hWndCombo = GetDlgItem(hDlg, IDC_FORMAT);
								i=SendMessage(hWndCombo,CB_GETCURSEL,NULL,0L);
								if (i==CB_ERR)
									{
									MessageBeep(100);
									}
								else
									{
	                        SetDlgItemText(hDlg, IDC_EDIT, FormatsArray[i].szFileMatch);
									nFileFormat=FormatsArray[i].nFormatID;
									// prepare directory update
									getcwd(szDefPath,127);
									lstrcat((LPSTR)szDefPath,(LPSTR)"\\");
									goto openfile;
									}
									
								break;

							}

						return(TRUE);
						break;

					case IDOK:
openfile:
              	GetDlgItemText(hDlg, IDC_EDIT, szOpenName, 128);
                 if (lg_strchr(szOpenName, '*') || lg_strchr(szOpenName, '?'))
							{
                 	SeparateFile( (LPSTR) str, (LPSTR) szDefSpec,(LPSTR) szOpenName);
                    // if (str[0])	*** this sometimes caused error. ***
                       lstrcpy(szDefPath, str);
							ChangeDefExt(szDefExt, szDefSpec);
                    UpdateListBox(hDlg);
                    return (TRUE);
                    }

						if (!szOpenName[0])
							{
                    MessageBox(hDlg,"No filename specified.",NULL,MB_OK|MB_ICONHAND);
                    return (TRUE);
                    }

						AddExt(szOpenName, szDefExt);

						// We have got a filename !
                 EndDialog(hDlg, hFile);
                 return (TRUE);

					case IDCANCEL:
                 EndDialog(hDlg, NULL);
                 return (FALSE);

				}

			break;

		case WM_INITDIALOG:
			getcwd(szDefPath,127);
			lstrcat((LPSTR)szDefPath,(LPSTR)"\\");
        UpdateListBox(hDlg);
        SetDlgItemText(hDlg, IDC_EDIT, szDefSpec);
        SendDlgItemMessage(hDlg,IDC_EDIT,EM_SETSEL,NULL,MAKELONG(0,0x7fff)); 

		   hWndCombo = GetDlgItem(hDlg, IDC_FORMAT);

			for (i = 0; *FormatsArray[i].szComboText; i++) // that is why nulltermination is important in FormatsArray!!!
				{
	  			SendMessage (hWndCombo,CB_ADDSTRING,0,(LONG)(LPSTR) FormatsArray[i].szComboText);
				}

			SendDlgItemMessage(hDlg,IDC_FORMAT,CB_SETCURSEL,0,0L);
			for (i = 0; *FormatsArray[i].szComboText; i++) 
				{
				if (nFileFormat == FormatsArray[i].nFormatID)
					{
					SendDlgItemMessage(hDlg,IDC_FORMAT,CB_SETCURSEL,i,0L);
					}
				}
        SetFocus(GetDlgItem(hDlg, IDC_EDIT));
        return (FALSE); 		// Indicates the focus is set to a control.
			break;

		}

	return(FALSE);
}



/*****************************************************************************
 Function
			SaveAsDlg

 Parameters
			normal dialog parameters

 Returns
			BOOL

 Description
			dialog function to specify "save as" filename

 Notes
			None.

 Author
			Lorincz_Gabor, 07/26/90 15:28
*****************************************************************************/

HANDLE FAR PASCAL SaveAsDlg(HWND hDlg, unsigned message, WORD wParam, LONG lParam)
{
    HANDLE hFile=1;     /* Temp value for return */
	 int i;
	 HWND hWndCombo;
	 int err;

    switch (message) {
        case WM_COMMAND:
            switch (wParam) {

					 case IDC_DIRLIST:
						switch (HIWORD(lParam))
							{
								case LBN_SELCHANGE:
                            /* If item is a directory name, append "*.*" */
                            if (DlgDirSelect(hDlg, str, IDC_DIRLIST))
										{
										  if (str[1]==':')
											{
												// if user would change to an other drive
												lstrcat( (LPSTR)str,(LPSTR)"\\");
 	                                 lstrcat((LPSTR)str, (LPSTR)szDefSpec);
											}
											else
												{
		                                lstrcat((LPSTR)str, (LPSTR)szDefSpec);
												}
										}
									 lstrcpy((LPSTR)szDefPath,(LPSTR)str);
                            SetDlgItemText(hDlg, IDC_EDIT, str);
                            SendDlgItemMessage(hDlg,
                                IDC_EDIT,
                                EM_SETSEL,
                                NULL,
                                MAKELONG(0, 0x7fff));
									break;
								case LBN_DBLCLK:
									goto openfile;
                            break;
							}
						break;

                case IDC_LISTBOX:
                    switch (HIWORD(lParam)) {

                        case LBN_SELCHANGE:
                            /* If item is a directory name, append "*.*" */
                            if (DlgDirSelect(hDlg, str, IDC_LISTBOX))
										{
                                lstrcat((LPSTR)str, (LPSTR)szDefSpec);
										}

                            SetDlgItemText(hDlg, IDC_EDIT, str);
                            SendDlgItemMessage(hDlg,
                                IDC_EDIT,
                                EM_SETSEL,
                                NULL,
                                MAKELONG(0, 0x7fff));
                            break;

                        case LBN_DBLCLK:
                            goto openfile;
                    }
                    return (TRUE);

					case IDC_FORMAT:
								switch (HIWORD(lParam))
									{
										case CBN_SELCHANGE:
										   hWndCombo = GetDlgItem(hDlg, IDC_FORMAT);
											i=SendMessage(hWndCombo,CB_GETCURSEL,NULL,0L);
											if (i==CB_ERR)
												{
													MessageBeep(100);
												}
											else
												{
				                          SetDlgItemText(hDlg, IDC_EDIT, FormatsArray[i].szFileMatch);
												  nFileFormat=FormatsArray[i].nFormatID;
													// prepare directory update
													getcwd(szDefPath,127);
													lstrcat((LPSTR)szDefPath,(LPSTR)"\\");

												  goto openfile;
												}
											break;
									}
								return(TRUE);
						break;

                case IDOK:
openfile:
                    GetDlgItemText(hDlg, IDC_EDIT, szOpenName, 128);
                    if (lg_strchr(szOpenName, '*') || lg_strchr(szOpenName, '?'))
							{
								// filename containes wildcards
                        SeparateFile( (LPSTR) str, (LPSTR) szDefSpec,(LPSTR) szOpenName);
                        // if (str[0]) **** This sometimes caused an error.
                            	lstrcpy(szDefPath, str);
                        ChangeDefExt(szDefExt, szDefSpec);
                        UpdateListBox(hDlg);
                        return (TRUE);
                    	}

                    if (!szOpenName[0]) {
                        MessageBox(hDlg, "No filename specified.",
                            NULL, MB_OK | MB_ICONHAND);
                        return (TRUE);
                    }

                    AddExt(szOpenName, szDefExt);

                    // we got a filename!!!!!!!!!!!!!!!!!!!!!!!!
                    EndDialog(hDlg, hFile);
                    return (TRUE);

						break;


                case IDCANCEL:
                    EndDialog(hDlg, NULL);
                    return (FALSE);
            }
            break;

        case WM_INITDIALOG:                        /* message: initialize    */
				SetDlgItemText(hDlg,IDOK,(LPSTR)"Save");;
				SetDlgItemText(hDlg,IDC_FILENAME,(LPSTR)"Save File Name:");
				SetWindowText(hDlg,(LPSTR)"Save File As");
				getcwd(szDefPath,127);
				lstrcat((LPSTR)szDefPath,(LPSTR)"\\");
            UpdateListBox(hDlg);
            SetDlgItemText(hDlg, IDC_EDIT, szDefSpec);
            SendDlgItemMessage(hDlg,               /* dialog handle      */
                IDC_EDIT,                          /* where to send message  */
                EM_SETSEL,                         /* select characters      */
                NULL,                              /* additional information */
                MAKELONG(0, 0x7fff));              /* entire contents      */

			   hWndCombo	  = GetDlgItem(hDlg, IDC_FORMAT);

				for (i=0; *FormatsArray[i].szComboText; i++) // that is why nulltermination is important in FormatsArray!!!
					{
					if(FormatsArray[i].nFormatID != ASCII_FORMAT)
		    			SendMessage (hWndCombo,CB_ADDSTRING,0,(LONG)(LPSTR) FormatsArray[i].szComboText);
					}

				SendDlgItemMessage(hDlg,IDC_FORMAT,CB_SETCURSEL,0,0L);
				for (i=0; *FormatsArray[i].szComboText; i++) 
					{
						if (nFileFormat==FormatsArray[i].nFormatID)
							{
								SendDlgItemMessage(hDlg,IDC_FORMAT,CB_SETCURSEL,i,0L);
							}
					}
            SetFocus(GetDlgItem(hDlg, IDC_EDIT));
            return (FALSE); /* Indicates the focus is set to a control */
				break;

    }
    return FALSE;
}



/*****************************************************************************
 Function
			DefaultFileFormatDlg

 Parameters
			normal dialog parameters

 Returns
			BOOL

 Description
			combobox to define default file format

 Notes
			SIDE EFFECT:
				Modifies lpPage->nFileFormat

 Author
			Lorincz_Gabor, 08/21/90 10:05
*****************************************************************************/

HANDLE FAR PASCAL DefaultFileFormatDlg(HWND hDlg, unsigned message, WORD wParam, LONG lParam)
{
	HWND hWndCombo;
	int i;

	switch (message)
		{
			case WM_COMMAND:
				switch (wParam)
					{
						case IDC_DEF_FORMAT:
								// combo box is activated by user
								switch (HIWORD(lParam))
									{
										case CBN_SELCHANGE:
										   hWndCombo = GetDlgItem(hDlg, IDC_DEF_FORMAT);
											i=SendMessage(hWndCombo,CB_GETCURSEL,NULL,0L);
											if (i==CB_ERR)
												{
													MessageBeep(100);
												}
											else
												{
				                          SetDlgItemText(hDlg, IDC_EDIT, FormatsArray[i].szFileMatch);
												  nFileFormat=FormatsArray[i].nFormatID;
												}
											break;

										case CBN_DBLCLK:
							           nDefaultFormat = nFileFormat;
   	                 			 EndDialog(hDlg, TRUE);
	                    	 return (TRUE);
											break;
									}
								return(TRUE);
							break;

						case IDCANCEL:
							// fall through
						case IDC_CANCEL:
                    	EndDialog(hDlg, NULL);
                    	return (FALSE);
							break;

						case IDOK:
							// fall through!!
						case IDC_OK:
							   nDefaultFormat = nFileFormat;
   	               EndDialog(hDlg, TRUE);
	               return (TRUE);
							break;
					}
				break;

			case WM_INITDIALOG:
				// set default file formatID
          nFileFormat = nDefaultFormat;

				// initialize combobox
			  hWndCombo	  = GetDlgItem(hDlg, IDC_DEF_FORMAT);

				for (i=0; *FormatsArray[i].szComboText; i++) // that is why nulltermination is important in FormatsArray!!!
					{
		    			SendMessage (hWndCombo,CB_ADDSTRING,0,(LONG)(LPSTR) FormatsArray[i].szComboText);
					}
				SendDlgItemMessage(hDlg,IDC_DEF_FORMAT,CB_SETCURSEL,0,0L);
				for (i=0; *FormatsArray[i].szComboText; i++) 
					{
						if (nFileFormat==FormatsArray[i].nFormatID)
							{
								SendDlgItemMessage(hDlg,IDC_DEF_FORMAT,CB_SETCURSEL,i,0L);
							}
					}
				break;
		}
	return (FALSE);
}



/*****************************************************************************
 Function
			lg_strchr()

 Parameters
			LPSTR in whhich search is processed
			char  char to find in string

 Returns
			LPSTR : valid if character found 
					  NULL if chhar not found

 Description
			searches chharacter in a null terminated string

 Notes
			STRING is expected to be NULL terminated

 Author
			Lorincz_Gabor, 06/08/90 09:21
*****************************************************************************/

char FAR *lg_strchr( LPSTR szString, char CharToFind)
{
	LPSTR lpReturn;

	lpReturn=szString;
	while ((*lpReturn)!='\0')
		{
			if (*lpReturn==CharToFind)
				{
					return(lpReturn);
				}
			lpReturn++;
		};
	return((LPSTR)NULL);

}


/****************************************************************************

    FUNCTION: UpdateDirList(HWND);

    PURPOSE: Update the list box of filenames in OpenDlg

****************************************************************************/

void UpdateDirList(HWND hDlg)
{
	int nSearchSuccess; // only for debugging reasons

    nSearchSuccess=DlgDirList(hDlg, (LPSTR)str, IDC_DIRLIST, IDC_PATH, 0x8010|0x4000);
}


/****************************************************************************

    FUNCTION: UpdateListBox(HWND);

    PURPOSE: Update the list box of filenames in OpenDlg

****************************************************************************/

void UpdateListBox(HWND hDlg)
{

    lstrcpy( (LPSTR) str,  (LPSTR) szDefPath);
    lstrcat( (LPSTR) str,  (LPSTR) szDefSpec);
    DlgDirList(hDlg, str, IDC_LISTBOX, IDC_PATH, 0x0000/*0x4010*/); // only files are searched
    SetDlgItemText(hDlg, IDC_EDIT, szDefSpec);
	 UpdateDirList(hDlg);
}


/****************************************************************************

    FUNCTION: ChangeDefExt(PSTR, PSTR);

    PURPOSE: Change the default extension

****************************************************************************/

void ChangeDefExt(LPSTR Ext, LPSTR Name)
{
    LPSTR pTptr;

    pTptr = Name;
    while (*pTptr && *pTptr != '.')
		{
        pTptr++;
		}
    if (*pTptr)
        if (!lg_strchr( (LPSTR) pTptr, '*') && !lg_strchr( (LPSTR) pTptr, '?'))
			{
            lstrcpy( (LPSTR) Ext,  (LPSTR) pTptr);
			}
}


/****************************************************************************

    FUNCTION: SeparateFile( LPSTR, LPSTR, LPSTR)

    PURPOSE: Separate filename and pathname

****************************************************************************/

void SeparateFile(LPSTR lpDestPath, LPSTR lpDestFileName, LPSTR lpSrcFileName)
{
    LPSTR lpTmp;
    char  cTmp;

    lpTmp = lpSrcFileName + (long) lstrlen(lpSrcFileName);
    while (*lpTmp != ':' && *lpTmp != '\\' && lpTmp > lpSrcFileName)
		{
        lpTmp = AnsiPrev(lpSrcFileName, lpTmp);
		}
    if (*lpTmp != ':' && *lpTmp != '\\')
	 	{
        lstrcpy(lpDestFileName, lpSrcFileName);
        lpDestPath[0] = 0;
        return;
    	}
    lstrcpy(lpDestFileName, lpTmp + 1);
    cTmp = *(lpTmp + 1);
    lstrcpy(lpDestPath, lpSrcFileName);
     *(lpTmp + 1) = cTmp;
    lpDestPath[(lpTmp - lpSrcFileName) + 1] = 0;
}



/****************************************************************************

    FUNCTION: AddExt(PSTR, PSTR);

    PURPOSE: Add default extension

/***************************************************************************/

void AddExt( LPSTR Name, LPSTR Ext)
{
    LPSTR pTptr;

    pTptr = Name;
    while (*pTptr && *pTptr != '.')
		{
        pTptr++;
		}
    if (*pTptr != '.')
		{
        lstrcat( Name, Ext);
		}
}

//--------------------------------- Footnotes ---------------------------------

//-------------------------------- End of file --------------------------------


