/****************************************************************************/
/* loadtiff.c  (c) Copyright Black Ice Software Inc.  1989 - 1991           */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/**********************************************************************************
  Module
	e:\c\tiffview\tiff\loadtiff.c

  Revision
		$Revision:   1.5  $

  Description
		Image decoding demo sources for TIFF.DLL .

  Notes
		Must link to TDEMO.EXE .

  History
	$Log:   E:/c/ujtdemo/loadtiff.c_v  $
 * 
 *    Rev 1.5   20 Feb 1991  9:30:00   Hrasko_Gabor
 * IBS image reader improved
 * 
 *    Rev 1.4   30 Jan 1991 11:38:18   Hrasko_Gabor
 * Functions are returning with boolean value.
 * They set gPage.hCurrentBitmap internaly.
 * 
 *    Rev 1.3   24 Jan 1991 10:55:14   Hrasko_Gabor
 * This is a reference version.
 * 
 *    Rev 1.2   15 Jan 1991 15:43:24   Hrasko_Gabor
 * 
 *    Rev 1.1   14 Jan 1991 15:30:22   Hrasko_Gabor
 * CCITT and IBS image handling routines.
 * 
 *    Rev 1.0   08 Jan 1991 19:11:32   Hrasko_Gabor
 * Initial revision.
**********************************************************************************/

#define CCITT_SUPPORT
#define IBS_SUPPORT
// #define TMSFAX_SUPPORT

//------------------------------- Include Files -------------------------------

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include "tiff.h"
#include "formats.h"
#include "data.h"
#include "tdemo.h"
#include "tiffdlg.h"

#ifdef IBS_SUPPORT
	#include "ibsdefs.h"
#endif

#ifdef TMSFAX_SUPPORT
	#include "tmsfax.h"
#endif

//----------------------------- External Functions ----------------------------

//----------------------------- External Variables ----------------------------

extern GPAGETYPE  gPage;
extern BOOL       bDecompG4;
extern HANDLE     hInst;
extern HPALETTE   hCurrentPal;

//------------------------------- Module Defines ------------------------------

#define PALETTESIZE 256
#define PALVERSION  0x300

// Macro for testing to fill a bitmap with black.
#define ERASEBITMAP(B,D,X,Y) 		 								\
				{					 														\
				HDC hTempDC;											  	\
				hTempDC = CreateCompatibleDC(D);			\
				if(hTempDC){													\
					SelectObject(hTempDC,B);						\
					BitBlt(hTempDC,0,0,(int)X,(int)Y,NULL,0,0,BLACKNESS);	\
					DeleteDC(hTempDC);									\
					}											 							\
				}

//-------------------------------- Module Types -------------------------------

//------------------------------ Global Functions -----------------------------

BOOL FAR PASCAL PutTiffIntoBitmap(LPSTR lpszFileName, int nImage);
BOOL FAR PASCAL LoadCcittImage(LPSTR lpszFileName, WORD wCompression);
HBITMAP FAR PASCAL LoadIBSImage(LPSTR lpszFileName);

//------------------------------ Global Variables -----------------------------

// This is a monochrome bitmap info structure. The dimensions must be
// defined during run-time. This variable is used by the SAVETIFF.C
// module as well.
BWBITMAPINFO BmInfo = {
		(DWORD)sizeof (BITMAPINFOHEADER),	// Type descriptor.
		0L,							 				//	Width.
		0L,											// Height.
		1,											// Planes.
		1,											// Bits per pixel.
		0L,0L,0L,0L,
		2L,											// Colors used.
		2L,				 							// Colors important.
		0,0,0,0,	  		 				// Black RGB.
		0xFF,0xFF,0xFF,0,				// White RGB.
		};

//------------------------------ Module Functions -----------------------------

int FAR PASCAL FlushTiffLine(LPSTR lpLineBuff, int nLine, LONG lParam);
int FAR PASCAL FlushCcittLine(LPSTR lpLineBuff, int nLine, LONG lParam);
static BOOL LoadTiff(TIFFFILE hChain, int nImage);
static HBITMAP DefineBitmap(HANDLE hBitmapInfo, HDC hDC);

//------------------------------ Module Variables -----------------------------

static HBITMAP hBitmap = NULL;
static HANDLE hBitmapInfo;
static LPBITMAPINFO lpBmI;
static HDC hDC;
static HCURSOR hOldCur;
static int nActLine;

static RECT rCli;
//---------------------------- Module Code (Globals) --------------------------



/*****************************************************************************
 Function
			PutTiffIntoBitmap

 Parameters
			lpszFileName				TIFF file name.
			nImage						Index of image in the file.

 Returns
			TRUE on success, FALSE if failed. The bitmap is returned theough
			gPage.hCurrentBitmap directly.

 Description
			Loads the nImage-th image from the TIFF file lpszFileName into a
			bitmap.

 Notes
			None.

 Author
			Hrasko_Gabor, 10/23/90 13:43
*****************************************************************************/

BOOL FAR PASCAL PutTiffIntoBitmap(LPSTR lpszFileName, int nImage)
{
	TIFFFILE hChain;
	FARPROC lpDlgFn;
	DWORD dwDlgParam;
	BOOL bDlgOk, bRetVal;

	bRetVal = FALSE;
	bDlgOk = TRUE;
	hBitmap = NULL;

	// Opens the TIFF file.
	hChain = OpenTiffFile(lpszFileName,T_READ);
	if (hChain != NULL)	  								// Opening OK.
		{
		// Registers the nImage-th image.
		if (GetTiffImage(hChain, nImage))				// Registering OK.
			{
			lpDlgFn = MakeProcInstance(TiffInfoDlgFn,hInst);
			dwDlgParam = ((DWORD)hChain << 16) + nImage;
			bDlgOk = DialogBoxParam(hInst,(LPSTR)"TiffInfo",NULL,lpDlgFn,dwDlgParam);
			FreeProcInstance(lpDlgFn);
			if(bDlgOk)
				{
				// Loads the TIFF image into hBitmap.
				bRetVal = LoadTiff(hChain, nImage);
				}	// Loading confirmed.

			}	// Endif registering image OK.

		// Closes TIFF file.
		CloseTiffFile(hChain);

		}	// Endif opening TIFF file OK.

	if(bDlgOk && !bRetVal)			// Error occured.
		{
		MessageBox(NULL,
					  (LPSTR)"Unable to access data. Loading has failed.",
					  (LPSTR)"TIFF error",
					  MB_OK|MB_ICONEXCLAMATION);
		}

	return(bDlgOk && bRetVal);
}


#ifdef CCITT_SUPPORT

/*****************************************************************************
 Function
			LoadCcittImage

 Parameters
			lpszFileName				Image file name.
			wMethod						Compression method.

 Returns
			TRUE on success, FALSE if failed. The bitmap is returned theough
			gPage.hCurrentBitmap directly.

 Description
			Loads a CCITT compressed image into a bitmap.

 Notes
			None.

 Author
			Hrasko_Gabor, 10/23/90 13:43
*****************************************************************************/

BOOL FAR PASCAL LoadCcittImage(LPSTR lpszFileName, WORD wCompression)
{
	int nFile;
	DWORD dwMethod;
	BOOL bSuccess;
	HWND hWnd;
	FARPROC lpfnLineProc, lpDlgFunc;

	switch(wCompression){
		case CCIT_31D_NOEOL_FORMAT:
				dwMethod = TIFF_CCITT;
				break;
		case CCIT_31D_EOL_FORMAT: 
				dwMethod = CCITT_GROUP3;
				break; 
		case CCIT_32_FORMAT: 
				dwMethod = MAKELONG(CCITT_GROUP3,CCITT_2D);
				break; 
		case CCIT_4_FORMAT: 
				dwMethod = CCITT_GROUP4;
				break; 
		case UNCOMPRESSED_FORMAT: 
				dwMethod = NOCOMPRESSION;
				break; 
		default:
				return(NULL); 
	}

	hWnd = GetActiveWindow();

	// Calls a dialog box to ask for image dimensions.
    lpDlgFunc = MakeProcInstance((FARPROC)AskDimsDlg,hInst);
	bSuccess = DialogBox(hInst,"AskDimsDlg",hWnd,lpDlgFunc);
	FreeProcInstance(lpDlgFunc);

	if(!bSuccess)
		{
		return(FALSE);
		}

	// Displays an hourglass-cursor and saves the original one.
	hOldCur = SetCursor(LoadCursor(NULL,IDC_WAIT));

	// Erases previous bitmap if any. 
	// It is neccessary to free the memory it occupies.
//	DeleteBitmap(TRUE);
	hBitmap = NULL;

	// Opens the image file.
	nFile = _lopen(lpszFileName,READ);
	if (nFile > 0)	  								// Opening OK.
		{
	
		// Gets a DC to be used later in the call back function.
		hDC = GetDC(hWnd);

		if (hDC != NULL)
			{

			hBitmap = CreateBitmap((int)BmInfo.bmiHeader.biWidth,
								   (int)BmInfo.bmiHeader.biHeight,
								   1,1,(LPSTR)NULL);
			if (hBitmap != NULL)
				{

				ERASEBITMAP(hBitmap,hDC,BmInfo.bmiHeader.biWidth,BmInfo.bmiHeader.biHeight);

				// Prepares callback function's pocedure instance.
				lpfnLineProc = MakeProcInstance(FlushCcittLine,hInst);

				// Fills the image in the bitmap.
				if(!DecodeCcittImage(nFile,
									 (WORD)BmInfo.bmiHeader.biWidth,
									 (WORD)BmInfo.bmiHeader.biHeight,
									 dwMethod,0L,lpfnLineProc))
					{
					DeleteObject(hBitmap);
					hBitmap = NULL;
					}

				// Frees the process instance.
				FreeProcInstance(lpfnLineProc);

				}	// Endif bitmap creation OK.

			ReleaseDC(hWnd,hDC);

			}	// Endif retrieving DC OK.

		// Closes image file.
		_lclose(nFile);
	}

	gPage.hCurrentBitmap = hBitmap;

	// Restores original cursor from hourglass-cursor.
	SetCursor(hOldCur);

	if(hBitmap == NULL)			// Error occured.
		{
		MessageBox(NULL,
					  (LPSTR)"Unable to access data. Loading has failed.",
					  (LPSTR)"TIFF error",
					  MB_OK|MB_ICONEXCLAMATION);
		}

	return(hBitmap != NULL);		 
}


#endif		// CCITT_SUPPORT


#ifdef IBS_SUPPORT

/*****************************************************************************
 Function
			LoadIBSImage

 Parameters
			lpszFileName				Image file name.

 Returns
			TRUE on success, FALSE if failed. The bitmap is returned theough
			gPage.hCurrentBitmap directly.

 Description
			Loads a IBS style image into a bitmap.

 Notes
			None.

 Author
			Hrasko_Gabor, 10/23/90 13:43
*****************************************************************************/

HBITMAP FAR PASCAL LoadIBSImage(LPSTR lpszFileName)
{
	int nFile;
	IBSIMGHEADER IBSInfo;
	DWORD dwMethod;
	HWND hWnd;
	FARPROC lpfnLineProc;

	// Displays an hourglass-cursor and saves the original one.
	hOldCur = SetCursor(LoadCursor(NULL,IDC_WAIT));

	// Erases previous bitmap if any. 
	// It is neccessary to free the memory it occupies.
//	DeleteBitmap(TRUE);
	hBitmap = NULL;

	// Opens the image file.
	nFile = _lopen(lpszFileName,READ);
	if (nFile > 0)	  								// Opening OK.
		{
	
		if((_lread(nFile,(LPSTR)&IBSInfo,sizeof(IBSIMGHEADER)) != -1) &&
		   (_llseek(nFile,IBSIMGOFFSET,SEEK_SET) != -1))
			{	

			switch(IBSInfo.wCompression){
				case IBS_RAW:
						dwMethod = NOCOMPRESSION;
						break;
				case IBS_G3NOEOL:
						dwMethod = TIFF_CCITT;
						break;
				case IBS_G31D: 
						dwMethod = CCITT_GROUP3;
						break; 
				case IBS_G32D: 
						dwMethod = MAKELONG(CCITT_GROUP3,CCITT_2D);
						break; 
				case IBS_G4: 
						dwMethod = CCITT_GROUP4;
						break; 
				default:
						_lclose(nFile);
						return(NULL); 
				}

			BmInfo.bmiHeader.biHeight = (DWORD)IBSInfo.nHeight; 
			BmInfo.bmiHeader.biWidth = (DWORD)IBSInfo.nWidth;
		
			// Gets a DC to be used later in the call back function.
			hWnd = GetActiveWindow();
			hDC = GetDC(hWnd);
		
			if (hDC != NULL)
				{
		
				hBitmap = CreateBitmap((int)BmInfo.bmiHeader.biWidth,
									   (int)BmInfo.bmiHeader.biHeight,
									   1,1,(LPSTR)NULL);
				if (hBitmap != NULL)
					{
		
					ERASEBITMAP(hBitmap,hDC,BmInfo.bmiHeader.biWidth,BmInfo.bmiHeader.biHeight);

					// Prepares callback function's pocedure instance.
					lpfnLineProc = MakeProcInstance(FlushCcittLine,hInst);
		
					// Fills the image in the bitmap.
					if(!DecodeCcittImage(nFile,
										 (WORD)BmInfo.bmiHeader.biWidth,
										 (WORD)BmInfo.bmiHeader.biHeight,
										 dwMethod,0L,lpfnLineProc))
						{
						//DeleteObject(hBitmap);
						//hBitmap = NULL;
						}
		
					// Frees the process instance.
					FreeProcInstance(lpfnLineProc);
		
					}	// Endif bitmap creation OK.
		
				ReleaseDC(hWnd,hDC);
		
				}	// Endif retrieving DC OK.
		
			}	// Reading IBS header OK.

		// Closes image file.
		_lclose(nFile);

		}	// Opening IBS image file OK.

	gPage.hCurrentBitmap = hBitmap;

	// Restores original cursor from hourglass-cursor.
	SetCursor(hOldCur);

	if(hBitmap == NULL)			// Error occured.
		{
		MessageBox(NULL,
					  (LPSTR)"Unable to access data. Loading has failed.",
					  (LPSTR)"TIFF error",
					  MB_OK|MB_ICONEXCLAMATION);
		}

	return(hBitmap != NULL);
}


#endif		// IBS_SUPPORT

//---------------------------- Module Code (Locals) ---------------------------


/**********************************************************************************
  Function
		FlushTiffLine

  Parameters
		lpLineBuff			Bits of uncompressed line inDIB format.
		nLine					Line index counted from zero.
		lParam				User parameter passed unchanged.

  Returns
		Error status. TOK if OK.

  Description
		Flushes the DIB type data buffer into a bitmap.

  Notes
		The wUsage parameter in SetDIBits() is constant in this version,
		because palette manipulation is commented out.

  Author
		Hrasko_Gabor,	12/26/90		Initial version.
**********************************************************************************/

int FAR PASCAL FlushTiffLine(LPSTR lpLineBuff, int nLine, LONG lParam)
{
WORD wUsage;
LPBITMAPINFO lpBmI;
TIFFLINEDATA far * lpTiffLineData = (TIFFLINEDATA far *)lParam;

	lpBmI = (LPBITMAPINFO) GlobalLock(lpTiffLineData->hBitmapInfo);
	wUsage = (lpBmI->bmiHeader.biClrUsed==0) ? DIB_RGB_COLORS : DIB_PAL_COLORS;

	// Writes data into bitmap. wUsage is not used in this version !!!
	SetDIBits(lpTiffLineData->hDC,lpTiffLineData->hBitmap,
              (int)lpBmI->bmiHeader.biHeight - ++nLine,1,
      			 	lpLineBuff, (LPBITMAPINFO)lpBmI, DIB_RGB_COLORS);

	GlobalUnlock(lpTiffLineData->hBitmapInfo);						  

	return(TOK);
}


/**********************************************************************************
  Function
		FlushCcittLine

  Parameters
		lpLineBuff			Bits of uncompressed line inDIB format.
		nLine					Line index counted from zero.
		lParam				User parameter passed unchanged.

  Returns
		Error status. TOK if OK.

  Description
		Flushes the DIB type data buffer into a bitmap.

  Notes
		None.

  Author
		Hrasko_Gabor,	12/26/90		Initial version.
**********************************************************************************/

int FAR PASCAL FlushCcittLine(LPSTR lpLineBuff, int nLine, LONG lParam)
{
	// Saves last processed line index.
	nActLine = nLine;

	// Writes data into bitmap. 
	SetDIBits(hDC,hBitmap,(int)BmInfo.bmiHeader.biHeight - ++nLine,1,
			 	lpLineBuff,(LPBITMAPINFO)&BmInfo,DIB_RGB_COLORS);

	return(TOK);
}


/**********************************************************************************
  Function
		LoadTiff

  Parameters
		hChain				Image chain handle.
		nImage				Image index.

  Returns
		TRUE on success, FALSE if failed. The bitmap is returned through
		gPage.hCurrentBitmap directly.

  Description
		Read a TIFF file into a bitmap.

  Notes
		None.

  Author
		Hrasko_Gabor,	12/28/90		Initial version.
**********************************************************************************/

static BOOL LoadTiff(TIFFFILE hChain, int nImage)
{
FARPROC lpfnLineProc;
TIFFLINEDATA TiffLineData;

	// Displays an hourglass-cursor and saves the original one.
	hOldCur = SetCursor(LoadCursor(NULL,IDC_WAIT));

	TiffLineData.hBitmap = NULL;

	// Retrieves color and dimensions info.
	TiffLineData.hBitmapInfo = GetTiffRGBQuad(hChain,nImage);

	if (TiffLineData.hBitmapInfo != NULL)
		{

		SetTiffError(TNOTENOUGHMEMORY);		// Error flag default value.

		// Gets a DC to be used later in the call back function.
		TiffLineData.hDC = GetDC(NULL);

		if (TiffLineData.hDC != NULL)
			{

			TiffLineData.hBitmap = DefineBitmap(TiffLineData.hBitmapInfo,
                                            TiffLineData.hDC);
			if (TiffLineData.hBitmap != NULL)
				{
          DWORD  dwValue;
          DWORD  dwRowsPerStrip;
          int    dwCompression;
          HANDLE hStr;
          RECT   rect;
          int    height;
          int    wRows = 0;

          GetTiffImageInfo(hChain,nImage,COMPRESSION,
                           (DWORD FAR*)&dwCompression);

          // Partial decopression 
          if (bDecompG4 && dwCompression == CCITT_GROUP4) {

        	   GetTiffImageInfo(hChain,nImage,ROWSPERSTRIP,
                              (DWORD FAR*)&dwRowsPerStrip);

             GetTiffImageInfoBuffer(hChain,nImage,STRIPOFFSETS,
                                    (HANDLE FAR*)&hStr,NULL,(DWORD FAR*)&dwValue);

             GetClientRect(gPage.hWndMain,  &rect);
             ClientToScreen(gPage.hWndMain, (POINT FAR *)&rect);
             ClientToScreen(gPage.hWndMain, ((POINT FAR *)&rect) + 1);
             height = rect.bottom - rect.top;

             wRows = (dwRowsPerStrip * dwValue);
             if (height < wRows) {

              	LPBITMAPINFO lpBmI;
               
                dwValue = height / dwRowsPerStrip;

                if (height % dwRowsPerStrip) dwValue++;
                // actual number of rows decompressed
                wRows = dwRowsPerStrip * dwValue;
                // saved the decompressed lines if image larger then screen
              	// Creates an empty BITMAP.

                // Creates an empty BITMAP.
              	lpBmI = (LPBITMAPINFO) GlobalLock(TiffLineData.hBitmapInfo);

                if (wRows < lpBmI->bmiHeader.biHeight) {
                   // save parameters for more decompression

                   gPage.dDecomp.hChain        =hChain;        // Image handle,
                   gPage.dDecomp.nImage        =nImage;        // image in the chain (zero based)
                   gPage.dDecomp.dwCompression =dwCompression; // Decompression type
                   gPage.dDecomp.dwRowsPerStrip=dwRowsPerStrip;// Number of rows in a strip.
                   gPage.dDecomp.wLinesDecomp  =wRows;         // Lines decompressed
                   gPage.dDecomp.nImgHeight    =lpBmI->bmiHeader.biHeight; // Image height
                   gPage.hBitmapInfo           =TiffLineData.hBitmapInfo;   // Bitmap info for decomp
                }
              	GlobalUnlock(TiffLineData.hBitmapInfo);
             }
             else {
                wRows = 0;
             }
          }

          // Prepares callback function's pocedure instance.
          lpfnLineProc = MakeProcInstance(FlushTiffLine,hInst);

				// Fills the image in the bitmap.
				if(!DecodeTiffImage(hChain,nImage,0,wRows,
                              (LONG)(TIFFLINEDATA far *)&TiffLineData,
                              lpfnLineProc))
					{
					if (TiffError() == TENDOFDATA)
						{
						MessageBox(NULL,
        							  (LPSTR)"Image was smaller that was defined.",
			    		  		  (LPSTR)"TIFF warning",
					      		  MB_OK|MB_ICONEXCLAMATION);
						}
					else				// real error.
						{
						DeleteObject(TiffLineData.hBitmap);
						TiffLineData.hBitmap = NULL;
              memset(&gPage.dDecomp, '\0', sizeof gPage.dDecomp);
						}
					}

				// Frees the process instance.
				FreeProcInstance(lpfnLineProc);

				}	// Endif bitmap creation OK.

			// Releases DC used in the callback function.
			ReleaseDC(NULL,TiffLineData.hDC);

			}	// Endif retrieving DC OK.

      // when line number is 0 the images is decompress
      if (!gPage.dDecomp.wLinesDecomp)
         GlobalFree(TiffLineData.hBitmapInfo);

		}	// Endif retrieving color and dimensions info OK.

	gPage.hCurrentBitmap = TiffLineData.hBitmap;

	// Restores original cursor from hourglass-cursor.
	SetCursor(hOldCur);

	return(TiffLineData.hBitmap != NULL);
}


/**********************************************************************************
  Function
		DefineBitmap

  Parameters
		hBitmapInfo				Handle of a memory block of BITMAPINFO type.

  Returns
		Bitmap handle or NULL in case of any error.

  Description
		Creates a bitmap based on data in hBitmapInfo.

  Notes
		None.

  Author
		Hrasko_Gabor,	01/08/91		Initial version.
**********************************************************************************/

static HBITMAP DefineBitmap(HANDLE hBitmapInfo, HDC hDC)
{
	HBITMAP hBitmap;
	LPBITMAPINFO lpBmI;

	// Creates an empty BITMAP.
	lpBmI = (LPBITMAPINFO) GlobalLock(hBitmapInfo);

	if (lpBmI->bmiHeader.biClrUsed == 2)  		// Monochrome image.
		{
		hBitmap = CreateBitmap((int)lpBmI->bmiHeader.biWidth,
									  (int)lpBmI->bmiHeader.biHeight,
									  1,1,(LPSTR)NULL);
		}
	else													// Color image.
		{
		hBitmap = CreateCompatibleBitmap(hDC,
											(int)lpBmI->bmiHeader.biWidth,
											(int)lpBmI->bmiHeader.biHeight);
		}

	GlobalUnlock(hBitmapInfo);

	return(hBitmap);
}


//--------------------------------- Footnotes ---------------------------------

//-------------------------------- End of file --------------------------------
