/****************************************************************************/
/* savetiff.c  (c) Copyright Black Ice Software Inc.  1989 - 1991           */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/**********************************************************************************
  Module
	e:\c\tiffview\tiff\savetiff.c

  Revision
		$Revision:   1.3  $

  Description
		Image encoding demo sources for TIFF.DLL .

  Notes
		Must link to TDEMO.EXE .

  History
	$Log:   E:/c/ujtdemo/savetiff.c_v  $
 * 
 *    Rev 1.3   20 Feb 1991  9:32:40   Hrasko_Gabor
 * Minor changes.
 * 
 *    Rev 1.2   24 Jan 1991 10:55:38   Hrasko_Gabor
 * This is a reference version.
 * 
 *    Rev 1.1   14 Jan 1991 12:26:32   Hrasko_Gabor
 * Palette functions are changed. 
 * 
 *    Rev 1.0   06 Jan 1991 20:09:28   Hrasko_Gabor
 * Initial revision.
**********************************************************************************/

//------------------------------- Include Files -------------------------------

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys\types.h>
#include <sys\timeb.h>
#include <windows.h>
#include "tiff.h"
#include "tiffdlg.h"
#include "data.h"
#include "formats.h"
#include "tools.h"

//----------------------------- External Functions ----------------------------

//----------------------------- External Variables ----------------------------

extern HANDLE hInst;
extern HPALETTE hCurrentPal;
extern BWBITMAPINFO BmInfo;		// Monochrome BITMAPINFO from LOADTIFF.C

//------------------------------- Module Defines ------------------------------

#define PALETTESIZE 256

//-------------------------------- Module Types -------------------------------

//------------------------------ Global Functions -----------------------------

BOOL FAR PASCAL SaveBitmapInTiffFormat(LPSTR lpszFileName, HBITMAP hBm);
BOOL FAR PASCAL SaveBitmapInCcittFormat(LPSTR lpszFileName, HBITMAP hBm, WORD wCompression);

//------------------------------ Global Variables -----------------------------

//------------------------------ Module Functions -----------------------------

int FAR PASCAL FillTiffLine(LPSTR lpLineBuff, int nLine, LONG lParam);
int FAR PASCAL FillCcittLine(LPSTR lpLineBuff, int nLine, LONG lParam);
static BOOL DefineCompressionInfo(TIFFFILE hChain, int nImage, WORD wCompMode);
static BOOL GetSystemPalette(WORD FAR * lpColor);
static BOOL DefineColorInfo(TIFFFILE hChain, int nImage);

//------------------------------ Module Variables -----------------------------

static BITMAPINFOHEADER BIH;
static HANDLE hBitmapInfo;
static LPBITMAPINFO lpBmI;
static HDC hDC;
static HBITMAP hBm;
static HCURSOR hOldCur;

WORD wCompMode;
char Date[25], Title[100], Artist[100];

int nClrsImprt;
int nPhotomInterp;

//---------------------------- Module Code (Globals) --------------------------


/*****************************************************************************
 Function
			SaveBitmapInTiffFormat

 Parameters
			lpszFileName					Tiff file name.
			hBitmap							Bitmap to convert into TIFF.
			wCompression					Compression type to use.
			nByteOrder						Byte order to use.

 Returns
			TRUE on success, otherwise FALSE.

 Description
			Convert bitmap 'hBitmap' into a TIFF file named 'lpszFileName'
			using compression algorithm defined in 'wCompression'.
			
 Notes
		 	It is a sample source. It is not part of the DLL.

 Author
			Hrasko_Gabor, 12/22/90 11:36
*****************************************************************************/

BOOL FAR PASCAL SaveBitmapInTiffFormat(LPSTR lpszFileName, HBITMAP hBitmap)
{
	TIFFFILE hChain;
	int nImage;
	FARPROC lpfnLineProc, lpDlgFunc;
	BOOL bSuccess = FALSE;
	HANDLE hWnd;
	BITMAP Bm;

	hBm = hBitmap;
	hWnd = GetActiveWindow();

	// Calls a dialog box to ask for a compression type and for other info-s.
	_fstrcpy(Title,lpszFileName);
  lpDlgFunc = MakeProcInstance((FARPROC)TiffDlg,hInst);
	bSuccess = DialogBoxParam(hInst,"SaveTiff",hWnd,lpDlgFunc,(LONG)hBm);
	FreeProcInstance(lpDlgFunc);

	#ifdef DEMO
		DEMO_MSG;
		return(FALSE);
	#endif

	if(!bSuccess)				  // 'Cancel' was pressed.
		{
		return(FALSE);
		}

	// Displays an hourglass-cursor and saves the original one.
	hOldCur = SetCursor(LoadCursor(NULL,IDC_WAIT));

	// Creates a TIFF image chain in the memory.
	hChain = OpenTiffFile((LPSTR)lpszFileName,T_APPEND);
	if(hChain != NULL)
		{
		// Defines byte order.
		if(ModiTiffFileInfo(hChain,NULL,LH_BYTE_ORDER))
			{
			// Registers a new image in the chain (it is the only one now).
			nImage = DefineTiffImage(hChain);
			if(nImage >= 0)
				{
				// Defines compression tags.
				if(DefineCompressionInfo(hChain,nImage,wCompMode))
					{
					// Define tags originated from the bitmap info.
					GetObject(hBitmap,sizeof(BITMAP),(LPSTR)&Bm);
					// Must change two fields if color image is saved as mono.
					if(nClrsImprt == 2)
						{
						Bm.bmBitsPixel = 1;
						Bm.bmPlanes = 1;
						}
					if(DefineGeneralTags(hChain,nImage,Bm) == TOK)
						{

						// Defines COLORMAP tag if neccessary.
						if(DefineColorInfo(hChain,nImage))
							{
	
							// Retrieves color and dimensions info for the call back
							// function indirectly from the tags defined before.
							hBitmapInfo = GetTiffRGBQuad(hChain,nImage);
						
							if (hBitmapInfo != NULL)
								{
						
								// Gets a DC to be used later in the call back function.
								hDC = GetDC(hWnd);
						
								if (hDC != NULL)
									{
	
									// Prepares callback function's pocedure instance.
									lpfnLineProc = MakeProcInstance(FillTiffLine,hInst);
									// Codes bitmap and writes data into TIFF file.
									bSuccess = EncodeTiffImage(hChain,nImage,
																		0L,lpfnLineProc);
									// Frees the process instance.
									FreeProcInstance(lpfnLineProc);
	
									// Releases DC used in the callback function.
									ReleaseDC(hWnd,hDC);
	
									}	// DC OK.
	
								GlobalFree(hBitmapInfo);
	
								}	// Bitmapinfo OK.
	
							}	// Endif general tags definition OK.

						}	// Endif color info definition OK.
	
					}	// Endif compression type definition OK.
	
				// Drops image. CloseTiffFile() will drop anyway.
				DropTiffImage(hChain,nImage);
				}	// Endif image registration OK.

			}	// Endif byte order definition OK.

		CloseTiffFile(hChain);
		}	// Endif image chain creation OK.

	// Restores original cursor from hourglass-cursor.
	SetCursor(hOldCur);

	if(!bSuccess)
		MessageBox(NULL,(LPSTR)"Saving has failed.",(LPSTR)"TIFF error",MB_OK|MB_ICONEXCLAMATION);

	return(bSuccess);
}



/**********************************************************************************
  Function
		SaveBitmapInCcittFormat

  Parameters
		None.

  Returns
		None.

  Description


  Notes
		None.

  Author
		Hrasko_Gabor,	01/10/91		Initial version.
**********************************************************************************/

BOOL FAR PASCAL SaveBitmapInCcittFormat(LPSTR lpszFileName, HBITMAP hBitmap, 
                 												WORD wCompression)
{
	int nFile;
	FARPROC lpfnLineProc;
	BOOL bSuccess = FALSE;
	HANDLE hWnd;
	BITMAP Bm;
	DWORD dwMethod;
	OFSTRUCT Dummy;

	#ifdef DEMO
		DEMO_MSG;
		return(FALSE);
	#endif

	// Saves in global.
	hBm = hBitmap;
	hWnd = GetActiveWindow();

	// Displays an hourglass-cursor and saves the original one.
	hOldCur = SetCursor(LoadCursor(NULL,IDC_WAIT));

	switch(wCompression){
		case CCIT_31D_NOEOL_FORMAT:
				dwMethod = TIFF_CCITT;
				break;
		case CCIT_31D_EOL_FORMAT: 
				dwMethod = CCITT_GROUP3 | CCITT_FILLBITS;
				break; 
		case CCIT_32_FORMAT: 
				dwMethod = MAKELONG(CCITT_GROUP3,CCITT_2D);
				break; 
		case CCIT_4_FORMAT: 
				dwMethod = CCITT_GROUP4;
				break; 
		case UNCOMPRESSED_FORMAT: 
				dwMethod = NOCOMPRESSION;
				break; 
		default:
				return(FALSE); 
	}
	
	// Opens a file for writing.
	nFile = OpenFile((LPSTR)lpszFileName,(LPOFSTRUCT)&Dummy,OF_CREATE|OF_WRITE);
	if(nFile > 0)
		{
						
		// Gets a DC to be used later in the call back function.
		hDC = GetDC(hWnd);
						
		if (hDC != NULL)
			{
	
			// Retrieves dimensions.
			GetObject(hBitmap,sizeof(BITMAP),(LPSTR)&Bm);
			BmInfo.bmiHeader.biWidth = (DWORD)Bm.bmWidth;
			BmInfo.bmiHeader.biHeight = (DWORD)Bm.bmHeight;

			// Prepares callback function's pocedure instance.
			lpfnLineProc = MakeProcInstance(FillCcittLine,hInst);
			// Codes bitmap and writes data into TIFF file.
			bSuccess = EncodeCcittImage(nFile,
												 Bm.bmWidth,
												 Bm.bmHeight,
												 dwMethod,
												 0L,
												 lpfnLineProc);
			// Frees the process instance.
			FreeProcInstance(lpfnLineProc);
	
			// Releases DC used in the callback function.
			ReleaseDC(hWnd,hDC);
	
			}	// DC OK.
	
		_lclose(nFile);
		}	// Endif image chain creation OK.

	// Restores original cursor from hourglass-cursor.
	SetCursor(hOldCur);

	if(!bSuccess)
		MessageBox(NULL,(LPSTR)"Saving has failed.",(LPSTR)"TIFF error",MB_OK|MB_ICONEXCLAMATION);

	return(bSuccess);
}


//---------------------------- Module Code (Locals) ---------------------------




/**********************************************************************************
  Function
		GetSystemPalette

  Parameters
		None.

  Returns
		None.

  Description


  Notes
		None.

  Author
		Hrasko_Gabor,	01/04/91		Initial version.
**********************************************************************************/

static BOOL GetSystemPalette(WORD FAR * lpColor)
{
	HANDLE hPalStru, hPal, hOldPal;
	LPLOGPALETTE lpLogPal;
	COLORREF Cref;
	HDC hDC;
	int wIndex;
	BOOL bRetVal = FALSE;

	// Allocate enough memory for a logical palette with
	// nClrsImprt entries and set the size and version fields
  // of the logical palette structure.
	hPalStru =  GlobalAlloc (GMEM_MOVEABLE|GMEM_ZEROINIT,
				  					 sizeof(LOGPALETTE) + sizeof(PALETTEENTRY) * nClrsImprt );

	if(hPalStru == NULL)	return(NULL);		// Not enough memory.

	lpLogPal = (LPLOGPALETTE) GlobalLock(hPalStru);
	lpLogPal->palVersion    = 0x300;
  lpLogPal->palNumEntries = nClrsImprt;

  // fill in intensities for all palette entry colors. 
  for (wIndex = 0; wIndex < nClrsImprt; wIndex++) 
		{
		*((WORD FAR *) (&lpLogPal->palPalEntry[wIndex].peRed)) = wIndex;
		lpLogPal->palPalEntry[wIndex].peBlue  = 0;
		lpLogPal->palPalEntry[wIndex].peFlags = PC_EXPLICIT;
     }

	// Create a logical color palette according the information
	// in the LOGPALETTE structure.
	hPal = CreatePalette ((LPLOGPALETTE) lpLogPal);

	if(hPal != NULL)
		{
		hDC = GetDC(NULL);
		if(hDC != NULL)
			{
			// Selects and realizes palette.
			hOldPal = SelectPalette(hDC,hPal,FALSE);
			RealizePalette(hDC);
			// Reads the system palette entries.
			for( wIndex=0 ; wIndex<nClrsImprt ; wIndex++ )
				{
				Cref = GetNearestColor(hDC,PALETTEINDEX(wIndex));
				lpColor[wIndex] = GetRValue(Cref) << 8;
				lpColor[wIndex+nClrsImprt] = GetGValue(Cref) << 8;
				lpColor[wIndex+2*nClrsImprt] = GetBValue(Cref) << 8;
				}
	
			bRetVal = TRUE;
			if (hOldPal)	SelectPalette(hDC,hOldPal,FALSE);
			ReleaseDC(NULL,hDC);
			}
		}

	GlobalUnlock(hPalStru);
	GlobalFree(hPalStru);

	return(bRetVal);
}



/**********************************************************************************
  Function
		DefineColorInfo

  Parameters
		None.

  Returns
		None.

  Description


  Notes
		None.

  Author
		Hrasko_Gabor,	01/04/91		Initial version.
**********************************************************************************/

static BOOL DefineColorInfo(TIFFFILE hChain, int nImage)
{
	HANDLE hColorMap;
	WORD FAR *lpColor;
	BOOL bRetVal;

	bRetVal = DefineTiffImageInfo(hChain,nImage,PHOTOMETRICINTERPRETATION,
											SHORTTYPE,(WORD)nPhotomInterp);

	if(bRetVal && (nPhotomInterp == 3))
		{
		hColorMap = GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,nClrsImprt * 3 * 2);
		if(hColorMap != NULL)
			{
			lpColor = (WORD FAR*) GlobalLock(hColorMap);
			if(GetSystemPalette((WORD FAR*)lpColor))
				{
				bRetVal = DefineTiffImageInfoBuffer(hChain,nImage,
																COLORMAP,lpColor,
																SHORTTYPE,nClrsImprt*3);
				}
			GlobalUnlock(hColorMap);
			GlobalFree(hColorMap);
			}
		}

	return(bRetVal);
}

/**********************************************************************************
  Function
		DefineCompressionInfo

  Parameters
		hChain				Image chain handle.
		nImage				Image index.
		wCompMode			Compression mode as defined in COMPDIAL.H .

  Returns
		TRUE on success, FALSE if failed.

  Description
		Defines several tags.

  Notes
		None.

  Author
		Hrasko_Gabor,	01/02/91		Initial version.
**********************************************************************************/

static BOOL DefineCompressionInfo(TIFFFILE hChain, int nImage, WORD wCompMode)
{
	WORD wCompression, wAuxInfo;
	WORD wAuxTag = 0;
	RATIO XRes, YRes;
	static struct tm *today;
	static time_t ltime;
	HDC hDC;

	// Calculates date and time string.
	time(&ltime);
	today = localtime(&ltime);
	strftime(Date,25,"%Y:%m:%d %H:%M:%S",today);

	// Calculates the resolution tags.
	hDC = GetDC(NULL);
	if (hDC == NULL)	
		return(FALSE);
	XRes.dwNumer = GetDeviceCaps(hDC,LOGPIXELSX);
	YRes.dwNumer = GetDeviceCaps(hDC,LOGPIXELSY);
	XRes.dwDenom = YRes.dwDenom = 1;
	ReleaseDC(NULL,hDC);

	switch(wCompMode){
		case ID_NOCOMP: 
						wCompression = NOCOMPRESSION;
						break;
		case ID_PACKBITS: 
						wCompression = PACK_BITS;
						break;
		case ID_LZW_DIFF: 
						wAuxInfo = 2; 				// Horizontal differencing.
						wAuxTag = PREDICTOR;
		case ID_LZW: 
						wCompression = LZW_COMPRESSION;
						break;
		case ID_CCITTG2: 
						wCompression = TIFF_CCITT;
						break;
		case ID_CCITTG31D: 
						wCompression = CCITT_GROUP3 | CCITT_FILLBITS;
						break;
		case ID_CCITTG32D: 
						wAuxInfo = 1;						// Two dimensional coding.
						wAuxTag = GROUP3OPTIONS;
						wCompression = CCITT_GROUP3;
						break;
		case ID_CCITTG4: 
						wCompression = CCITT_GROUP4;
						break;
		default: 
						return(FALSE);
	}

	// Defines the tags. Resolution is measured in Inch (RESOLUTIONUNIT = 2).
	if(!DefineTiffImageInfo(hChain,nImage,COMPRESSION,SHORTTYPE,wCompression))	return(FALSE);
	if(!DefineTiffImageInfoBuffer(hChain,nImage,SOFTWARE,"TIFF View from Black Ice",ASCIITYPE,25))	return(FALSE);
	if(!DefineTiffImageInfoBuffer(hChain,nImage,DATETIME,Date,ASCIITYPE,_fstrlen(Date)+1))	return(FALSE);
	if(!DefineTiffImageInfoBuffer(hChain,nImage,IMAGEDESCRIPTION,Title,ASCIITYPE,_fstrlen(Title)+1))	return(FALSE);
	if(!DefineTiffImageInfoBuffer(hChain,nImage,ARTIST,Artist,ASCIITYPE,_fstrlen(Artist)+1))	return(FALSE);
	if(!DefineTiffImageInfo(hChain,nImage,RESOLUTIONUNIT,SHORTTYPE,2))	return(FALSE);
	if(!DefineTiffImageInfoBuffer(hChain,nImage,XRESOLUTION,(VOID FAR*)&XRes,RATIONALTYPE,1))	return(FALSE);
	if(!DefineTiffImageInfoBuffer(hChain,nImage,YRESOLUTION,(VOID FAR*)&YRes,RATIONALTYPE,1))	return(FALSE);

	// Defines auxillary compression info if neccessary.
	if((wAuxTag != 0) && (!DefineTiffImageInfo(hChain,nImage,wAuxTag,SHORTTYPE,wAuxInfo)))	return(FALSE);

	return(TRUE);
}



/**********************************************************************************
  Function
		FillTiffLine

  Parameters
		lpLineBuff			Bits of uncompressed line inDIB format.
		nLine					Line index counted from zero.
		lParam				User parameter passed unchanged.

  Returns
		Error status. TOK if OK.

  Description
		Fill the data buffer with a DIB line from the image.

  Notes
		None.

  Author
		Hrasko_Gabor,	12/26/90		Initial version.
**********************************************************************************/

int FAR PASCAL FillTiffLine(LPSTR lpLineBuff, int nLine, LONG lParam)
{
	lpBmI = (LPBITMAPINFO) GlobalLock(hBitmapInfo);

	// Reads data from bitmap. 
	GetDIBits(hDC,hBm,(int)lpBmI->bmiHeader.biHeight - ++nLine,1,
			 	lpLineBuff,(LPBITMAPINFO)lpBmI,DIB_PAL_COLORS);

	GlobalUnlock(hBitmapInfo);

	return(TOK);
}


/**********************************************************************************
  Function
		FillCcittLine

  Parameters
		lpLineBuff			Bits of uncompressed line inDIB format.
		nLine					Line index counted from zero.
		lParam				User parameter passed unchanged.

  Returns
		Error status. TOK if OK.

  Description
		Fill the data buffer with a DIB line from the image.

  Notes
		None.

  Author
		Hrasko_Gabor,	12/26/90		Initial version.
**********************************************************************************/

int FAR PASCAL FillCcittLine(LPSTR lpLineBuff, int nLine, LONG lParam)
{
	// Reads data from bitmap. 
	GetDIBits(hDC,hBm,(int)BmInfo.bmiHeader.biHeight - ++nLine,1,
			 	lpLineBuff,(LPBITMAPINFO)&BmInfo,DIB_RGB_COLORS);

	return(TOK);
}



//--------------------------------- Footnotes ---------------------------------

//-------------------------------- End of file --------------------------------

