/****************************************************************************/
/* snap.c    (c) Copyright Black Ice Software Inc.  1989 - 1991             */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/*****************************************************************************
 Module
			c:\test\blacky\snap.c

 Revision
			$Revision:   1.1  $

 Description
			functions to process Snapping

 Notes
			

 
   $Log:   E:/c/ujtdemo/snap.c_v  $
 * 
 *    Rev 1.1   28 Jan 1991 12:49:34   Hrasko_Gabor
 * Not through clipboard any more.
 * Outlook is improved.
 * 
 *    Rev 1.0   24 Jan 1991 10:55:12   Hrasko_Gabor
 * Initial revision.
 * 
 *    Rev 1.6   27 Aug 1990 14:30:28   Lorincz_Gabor
 * refinements on backtracking when not enough memory
 * 
 *    Rev 1.5   20 Aug 1990 10:06:00   Lorincz_Gabor
 * snapshotting was improved
 * when bitmap into which screen is received could not be created, a second
 * trial is done to create monochrome bitmap which demands less memory
 * 
 *    Rev 1.4   28 Jul 1990 20:15:18   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.3   25 Jul 1990 10:48:08   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.2   14 Jul 1990  8:32:20   Lorincz_Gabor
 * global variables in module were made static
 * 
 *    Rev 1.1   12 Jul 1990 16:11:06   Lorincz_Gabor
 * No change.
 * 
 *    Rev 1.0   11 Jul 1990  7:45:16   Lorincz_Gabor
 * Initial revision.
 * 
 *    Rev 1.0   06 Jul 1990  9:35:30   Lorincz_Gabor
 * Initial revision.
*****************************************************************************/

//------------------------------- Include Files -------------------------------

#include <stdlib.h>
#include <math.h>

#define NOMINMAX
#include <windows.h>

#include "data.h"
#include "tdemo.h" 

//----------------------------- External Functions ----------------------------

//----------------------------- External Variables ----------------------------

extern HANDLE    hInst;
extern HWND      hWndFrame;

//------------------------------- Module Defines ------------------------------

//-------------------------------- Module Types -------------------------------

//------------------------------ Global Functions -----------------------------

//------------------------------ Global Variables -----------------------------

//------------------------------ Module Functions -----------------------------

static BOOL CalcZoomArea(HWND hWnd);

//------------------------------ Module Variables -----------------------------

//---------------------------- Module Code (Globals) --------------------------


/**********************************************************************************
  Function
		Snap

  Parameters
		hWnd					Window handle.
		BOOL					TRUE if extended grab required.

  Returns
		None.

  Description
		Processes a whole grab manipulation.

  Notes
		None.

  Author
		Lorincz_Gabor
**********************************************************************************/

VOID Snap(HWND hWnd, BOOL bExtendGrab, GPAGETYPE *pPage)
{
	POINT pM;
	MSG msg;
	HDC hDC, hMemDC;
	HANDLE hBitmap;
	int x1,x2,y1,y2;
	BOOL bTrack = FALSE;

	if(bExtendGrab)
		{
		ShowWindow(hWndFrame, SW_SHOWMINIMIZED);
		UpdateWindow(hWndFrame);
		}

	hDC = CreateDC ("DISPLAY", NULL, NULL, NULL) ;

	if (hDC != NULL)
		{	
		SetCursor(LoadCursor(hInst,"Snap"));
		SetCapture(hWnd);
		}

	SetROP2(hDC, R2_NOTXORPEN);
	
	// Eats mouse messages until a WM_LBUTTONUP is encountered. 
	// Meanwhile continue to draw banding rectangle.
	do	{
		WaitMessage();

		if (PeekMessage(&msg,NULL,WM_MOUSEFIRST,WM_MOUSELAST,PM_REMOVE))
			{
			switch(msg.message)
				{
				case WM_LBUTTONDOWN:
					bTrack = TRUE;
					pM = MAKEPOINT(msg.lParam);
					ClientToScreen(hWnd,(LPPOINT)&pM);
					pPage->Rect.left = pPage->Rect.right = pM.x;
					pPage->Rect.top = pPage->Rect.bottom = pM.y;
					break;

				case WM_MOUSEMOVE:
					if(bTrack)
						{
						pM = MAKEPOINT(msg.lParam);
						ClientToScreen(hWnd,(LPPOINT)&pM);
						if((pM.x != pPage->Rect.right) || (pM.y != pPage->Rect.bottom))
							{
							Rectangle(hDC,pPage->Rect.left,pPage->Rect.top,pPage->Rect.right,pPage->Rect.bottom);
							pPage->Rect.right=pM.x;
							pPage->Rect.bottom=pM.y;
							Rectangle(hDC,pPage->Rect.left,pPage->Rect.top,pPage->Rect.right,pPage->Rect.bottom);
							}
						}
					break;

				}	// switch.
			}
	
		} while(msg.message != WM_LBUTTONUP);
	
	ReleaseCapture();
	
	// Deletes last rectangle.
	Rectangle(hDC,pPage->Rect.left,pPage->Rect.top,pPage->Rect.right,pPage->Rect.bottom);

	hBitmap = NULL;
	NORMRECT(pPage->Rect);
	x1 = pPage->Rect.left;
	x2 = pPage->Rect.right;
	y1 = pPage->Rect.top;
	y2 = pPage->Rect.bottom;

    hMemDC = CreateCompatibleDC (hDC);

	if (hMemDC != NULL)
		{
		hBitmap = CreateCompatibleBitmap(hDC,x2-x1,y2-y1);
		if (hBitmap == NULL)
			{
			// try to create a monochrome bitmap it need smaller space
			hBitmap = CreateBitmap(x2-x1,y2-y1,1,1,NULL);
			}

		if (hBitmap != NULL)
			{
	   	SelectObject (hMemDC, hBitmap) ;
   		BitBlt (hMemDC,0,0,x2-x1,y2-y1,hDC,x1,y1,SRCCOPY);
			}

		DeleteDC(hMemDC);
		}	// Endif compatible DC creation OK.

	// Deletes source DC.
    DeleteDC (hDC) ;

	if (hBitmap != NULL)
		ReplaceBitmap(hBitmap,TRUE,TRUE, pPage);
	else			  			// Some error occured.
		MessageBox(hWnd,(LPSTR)"Not enough memory to copy marked area.",
					(LPSTR)"SORRY",MB_ICONEXCLAMATION|MB_OK);
 
    // decompressed the image
    memset(&pPage->dDecomp, '\0', sizeof pPage->dDecomp);

	if(bExtendGrab)
		{
		ShowWindow(hWndFrame,SW_RESTORE);
		UpdateWindow(hWndFrame);
		}

}

/**********************************************************************************
  Function
		Zoom

  Parameters
		hWnd				Window handle.
		lStartPos		Position of cursor.

  Returns
		None.

  Description
		Selects a rectangular area from the image.

  Notes
		None.

  Author
		Hrasko_Gabor,	01/29/91		Initial version.
**********************************************************************************/

VOID Zoom(HWND hWnd, LONG lStartPos, GPAGETYPE *pPage)
{
	POINT pM;
	MSG msg;
	HDC hDC;
	
	hDC=GetDC(hWnd);
	pPage->Rect = (pPage->nDispFormat==DISP_PREVIEW) ? pPage->rWhole : pPage->rActDraw;
	SetWindowView(hDC, pPage);
	LPtoDP(hDC,(LPPOINT)&pPage->Rect,2);
	pM=MAKEPOINT(lStartPos);
	if(!PtInRect((LPRECT)&pPage->Rect,pM))
		{
		MessageBeep(0);
		ReleaseDC(hWnd,hDC);
		return;
		}

	ClientToScreen(hWnd,(LPPOINT)&pPage->Rect.left);
	ClientToScreen(hWnd,(LPPOINT)&pPage->Rect.right);
	ClipCursor((LPRECT)&pPage->Rect);
	SetCapture(hWnd);

	DPtoLP(hDC,&pM,1);
	pPage->Rect.right = pPage->Rect.left = pM.x;
	pPage->Rect.bottom = pPage->Rect.top = pM.y;
	SetROP2(hDC, R2_NOTXORPEN);
	
	// Eats mouse messages until a WM_LBUTTONUP is encountered. 
	// Meanwhile continue to draw banding rectangle.
	do	{
		WaitMessage();
		if (PeekMessage(&msg,NULL,WM_MOUSEFIRST,WM_MOUSELAST,PM_REMOVE))
			{
			pM = MAKEPOINT(msg.lParam);
			DPtoLP(hDC,&pM,1);
			if((pM.x != pPage->Rect.right) || (pM.y != pPage->Rect.bottom))
				{
				Rectangle(hDC,pPage->Rect.left,pPage->Rect.top,pPage->Rect.right,pPage->Rect.bottom);
				pPage->Rect.right=pM.x;
				pPage->Rect.bottom=pM.y;
				Rectangle(hDC,pPage->Rect.left,pPage->Rect.top,pPage->Rect.right,pPage->Rect.bottom);
				}
			}
	
		} while(msg.message != WM_LBUTTONUP);
	
	Rectangle(hDC,pPage->Rect.left,pPage->Rect.top,pPage->Rect.right,pPage->Rect.bottom);
	ReleaseCapture();
	ClipCursor((LPRECT)NULL);
	
	NORMRECT(pPage->Rect);

	// A vary small rectangle is interpreted as a bad trial.
	if (((pPage->Rect.right-pPage->Rect.left) <= 2) ||
		  ((pPage->Rect.bottom-pPage->Rect.top) <= 2))
		{
		ReleaseDC(hWnd,hDC);
		return;		   // empty zoom !
		}
	
	CopyRect((LPRECT)&pPage->rActDraw,(LPRECT)&pPage->Rect );
	pPage->nDispFormat = DISP_ZOOMED;
	// Calculates mapping mode with the new parameters.
	CalcWindowView(pPage);
	SetWindowView(hDC, pPage);
	// Adjusts rActDraw based on values really set by GDI.
	GetClientRect(hWnd,(LPRECT)&pPage->rActDraw);
	DPtoLP(hDC,(LPPOINT)&pPage->rActDraw,2);
	AdjustArea(pPage);
  	ReleaseDC(hWnd,hDC);
		
	InvalidateRect(hWnd,NULL,TRUE);
	UpdateWindow(hWnd);
	
	return;
}


//---------------------------- Module Code (Locals) ---------------------------

//--------------------------------- Footnotes ---------------------------------

//-------------------------------- End of file --------------------------------



