/****************************************************************************/
/* status.c   (c) Copyright Black Ice Software Inc.  1989 - 1991             */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/*****************************************************************************
 Module
         c:\test\tiff\status.c

 Revision
         $Revision:   1.4  $

 Description
         status display module of TIFF 5.0 demo application

 Notes
         

*****************************************************************************/ 
#include <windows.h>

#include "data.h"
#include "tdemo.h"

extern   HANDLE	hInst;
extern   int		xChar, yChar;

#define  IDTIMER	1

static   BOOL boolTimerUp;
static   int tCounter = 4;

HPEN 	   hPenWhite, hPenGray, hPenShadow;

VOID     ErrorMessage(HWND hWnd, HANDLE hInst, int ErrorId );
VOID     ShowStatus(HWND hWnd, int StringId, int Style);

static   VOID	DrawFrame(HWND hWnd);

//---------------------------------------------------------------------------
long FAR PASCAL StatusWndProc(HWND hWnd, unsigned message, WORD wParam, LONG lParam)
{																			  
	switch (message)
	{
		case WM_CREATE:
		{
			SetWindowWord (hWnd, GWW_DISPLAYSTYLE, ST_REGULAR);

			hPenWhite = CreatePen
			(
				PS_INSIDEFRAME, 1, RGB(255, 255, 255)
			);
			hPenGray = CreatePen
			(
				PS_INSIDEFRAME, 
				GetSystemMetrics(SM_CYDLGFRAME) + 1, GetSysColor(COLOR_BTNFACE)
			);
			hPenShadow = CreatePen
			(
				PS_INSIDEFRAME, 1, GetSysColor(COLOR_BTNSHADOW)
			);
		}
		break;

		case WM_DESTROY:
			DeleteObject(hPenWhite);
			DeleteObject(hPenGray);
			DeleteObject(hPenShadow);
		break;

#ifdef DEBUG
		case WM_SETTEXT:
		{
			HDC	hDC;
			RECT	rc;
			LPSTR	lpsz = (LPSTR)lParam;

			hDC = GetDC(hWnd);
			SetBkMode(hDC, TRANSPARENT);
			SetTextColor(hDC, GetSysColor(COLOR_WINDOWTEXT));
			TextOut(hDC, xChar, (rc.bottom - yChar) / 2, lpsz, lstrlen(lpsz));
			ReleaseDC(hWnd, hDC);
		}
		break;
#endif

		case PM_STATUSTEXT:
			SetWindowWord (hWnd, GWW_DISPLAYSTYLE, (int)wParam);
			SetWindowText(hWnd, (LPSTR)lParam);
			InvalidateRect(hWnd, NULL, TRUE);

			/*
				if it's a MENUSELECT text, or "Ready" message, 
				it has nothing to do with timers
			*/
			if (wParam == ST_MENU || ST_READY)
			{
				if (boolTimerUp)
				{
					KillTimer(hWnd, IDTIMER);
					tCounter = 4;
					boolTimerUp = FALSE;
				}

				return 0L;
			}
			else		/* REGULAR status msg or ALARM status msg */
			{
				/* if the timer is already going, reset the count */
				if (boolTimerUp)
					tCounter = 4;
				else
				{
					if (!SetTimer(hWnd, IDTIMER, 1000, NULL))
					 	ErrorMessage(hWnd, hInst, IDS_MANYCLOCKS);
					else
						boolTimerUp = TRUE;
				}
			}
		break;

		case WM_TIMER:
			if (tCounter == 0)
			{
				char szReady[32];

				memset(szReady, 0, 32);

				/* wipe out the message */
				KillTimer(hWnd, IDTIMER);
				LoadString(hInst, IDS_READY, szReady, 31);
				SetWindowText(hWnd, szReady);
				MessageBeep(0);
				SetWindowWord (hWnd, GWW_DISPLAYSTYLE, ST_REGULAR);
				InvalidateRect(hWnd, NULL, TRUE);
				tCounter = 4;
				boolTimerUp = FALSE;
				return 0L;
			}       

			tCounter-- ;
		break;

		case PM_STATUSBMP:
			SetWindowWord (hWnd, GWW_DISPLAYSTYLE, (int)wParam);
			SetWindowText(hWnd, "");
			InvalidateRect(hWnd, NULL, TRUE);
		break;

		case WM_NCPAINT:
			DrawFrame( hWnd );
		break;

		case WM_PAINT:
		{
			PAINTSTRUCT	ps;
			HDC			hDC, hMemDC = NULL;
			RECT		rc;
			char		szBuffer[128];
			WORD		wStyle = GetWindowWord (hWnd, GWW_DISPLAYSTYLE);

			memset(szBuffer, 0, 128);
			memset((LPSTR)&ps, 0, sizeof(PAINTSTRUCT));

			GetClientRect(hWnd, &rc);

			hDC = BeginPaint(hWnd, &ps);
			if (GetWindowText(hWnd, szBuffer, 128))
			{
				if (wStyle == ST_REGULAR || wStyle == ST_READY)
					SetTextColor(hDC, RGB(0, 0, 255));
				else if (wStyle == ST_ALARM)
					SetTextColor(hDC, RGB(255, 0, 0));
				else
					SetTextColor(hDC, RGB(0, 0, 0));

				SetBkMode(hDC, TRANSPARENT);
				TextOut(hDC, xChar, (rc.bottom - yChar) / 2, szBuffer, lstrlen(szBuffer));
			}
			else
			{
				HBITMAP	hBitmap = NULL;
				BITMAP	bm;

				memset((LPSTR)&bm, 0, sizeof(BITMAP));

				switch (wStyle)
				{
					case ST_COFFEE:
						hBitmap = LoadBitmap(hInst, MAKEINTRESOURCE(COFFEE));
					break;
				}

				if (hBitmap)
				{
					hMemDC = CreateCompatibleDC( hDC );
					SelectObject(hMemDC, hBitmap);
					GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);

					StretchBlt
					(
						hDC, (rc.right - bm.bmWidth) / 2, 0,
						rc.bottom, rc.bottom, 
						hMemDC, 0, 0, bm.bmWidth, bm.bmHeight,
						SRCCOPY
					);
					DeleteDC(hMemDC);
				}
			}

			EndPaint(hWnd, &ps);
		}
		break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}

	return 0L;
}

//---------------------------------------------------------------
VOID ShowStatus(HWND hWnd, int StringId, int Style)
{
	char szTemp[128];

	memset(szTemp, 0, 128);

	if (StringId != -1)
	{
		LoadString(hInst, StringId, szTemp, 127);
		SendMessage(hWnd, PM_STATUSTEXT, Style, (LONG)(LPSTR)szTemp);
		return;
	}
	else
		SendMessage(hWnd, PM_STATUSBMP, Style, 0L);
}

//-------------------------------------------------------------------
static VOID	DrawFrame(HWND hWnd)
{
	HDC	hDC = NULL;
	RECT	rc;
	int	iFrame = GetSystemMetrics(SM_CYDLGFRAME);

	GetWindowRect(hWnd, &rc);
	hDC = GetWindowDC(hWnd);

	SelectObject(hDC, GetStockObject(NULL_BRUSH));
	SelectObject(hDC, hPenGray);
	Rectangle(hDC, 0, 1, rc.right - rc.left, rc.bottom - rc.top);

	SelectObject(hDC, hPenWhite);
	MoveTo(hDC, rc.right - rc.left - iFrame, iFrame);
	LineTo(hDC, rc.right - rc.left - iFrame, rc.bottom - rc.top - iFrame);
	LineTo(hDC, iFrame, rc.bottom - rc.top - iFrame);

	SelectObject(hDC, hPenShadow);
	LineTo(hDC, iFrame, iFrame);
	LineTo(hDC, rc.right - rc.left - iFrame, iFrame);

	// the very upper white line
	SelectObject(hDC, hPenWhite);
	MoveTo(hDC, 0, 0);
	LineTo(hDC, rc.right - rc.left, 0);

	SelectObject(hDC, GetStockObject(NULL_PEN));
	ReleaseDC(hWnd, hDC);
}

//-------------------------------------------------------------------
VOID ErrorMessage(HWND hWnd, HANDLE hInst, int ErrorId )
{
	char szTemp[128], szAppName[64];

	memset((LPSTR)szTemp, 0, 128);
	memset((LPSTR)szAppName, 0, 64);

	if (hWnd == NULL)
		lstrcpy((LPSTR)szAppName, "");
	else
		LoadString(GetWindowWord(hWnd, GWW_HINSTANCE), IDS_APPNAME, (LPSTR)szAppName, 63);

 	LoadString(hInst, ErrorId, (LPSTR)szTemp, 127);
	MessageBox(hWnd, (LPSTR)szTemp, (LPSTR)szAppName, MB_OK|MB_ICONEXCLAMATION);
}

