/****************************************************************************/
/* tiffdlg.c  (c) Copyright Black Ice Software Inc.  1989 - 1991            */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/**********************************************************************************
  Module
		e:\c\ujtdemo\tiffdlg.c

  Revision
		$Revision:   1.1  $

  Description
		Information dialog box call back functions.

  Notes
		None.

  History
	$Log:   E:/c/ujtdemo/tiffdlg.c_v  $
 * 
 *    Rev 1.1   30 Jan 1991 11:46:32   Hrasko_Gabor
 * Predictor is not displayed in TiffInfoDlgFn() if format id CCITT G4.
 * A bug is fixed in TiffDlg().
 * 
 *    Rev 1.0   24 Jan 1991 10:56:20   Hrasko_Gabor
 * Initial revision.
**********************************************************************************/

//------------------------------- Include Files -------------------------------

#include <stdio.h>
#include <stdlib.h>	
#include <windows.h>
#include "data.h"
#include "tiff.h"
#include "tiffdlg.h"

//----------------------------- External Functions ----------------------------

//----------------------------- External Variables ----------------------------

extern GPAGETYPE gPage;
extern BWBITMAPINFO BmInfo;		// Monochrome BITMAPINFO from LOADTIFF.C
extern WORD wCompMode;
extern char Title[100], Artist[100];
extern int nClrsImprt;
extern int nPhotomInterp;

//------------------------------- Module Defines ------------------------------

#define UNDEF_HEIGHT 1000

//-------------------------------- Module Types -------------------------------

//------------------------------ Global Functions -----------------------------

//------------------------------ Global Variables -----------------------------

//------------------------------ Module Functions -----------------------------

//------------------------------ Module Variables -----------------------------

//---------------------------- Module Code (Globals) --------------------------



/**********************************************************************************
  Function
		TiffInfoDlgFn

  Parameters
		hDlg						Dialog box handle.

  Returns
		None.

  Description
		Callback function for Dialog box "TiffInfo".

  Notes
		None.

  Author
		Hrasko_Gabor,	01/29/91		Predictor not displayed at CCITT G4.
		Hrasko_Gabor,	01/17/91		Initial version.
**********************************************************************************/

int FAR PASCAL TiffInfoDlgFn(HWND hDlg, WORD wMsg, WORD wParam, DWORD lParam)
{
	TIFFFILE hChain;
	int nImage;
	DWORD dwValue, dwBits;
	char Buff[30];
	HANDLE hStr;
	WORD wPredTag = 0;
	TIFFHDR Info;

	switch(wMsg){

		case WM_INITDIALOG:

			hChain = HIWORD(lParam);
			nImage = LOWORD(lParam);

			GetTiffFileInfo(hChain,(TIFFHDR FAR*)&Info);
			SetWindowText(hDlg,(LPSTR)Info.cFileName); 

			if(GetTiffImageInfoBuffer(hChain,nImage,IMAGEDESCRIPTION,(HANDLE FAR*)&hStr,NULL,NULL))
				{
				SetDlgItemText(hDlg,ID_TIFFTITLE,(LPSTR)GlobalLock(hStr));
				GlobalUnlock(hStr);
				GlobalFree(hStr);
				}

			if(GetTiffImageInfoBuffer(hChain,nImage,ARTIST,(HANDLE FAR*)&hStr,NULL,NULL))
				{
				SetDlgItemText(hDlg,ID_TIFFARTIST,(LPSTR)GlobalLock(hStr));
				GlobalUnlock(hStr);
				GlobalFree(hStr);
				}

			if(GetTiffImageInfoBuffer(hChain,nImage,DATETIME,(HANDLE FAR*)&hStr,NULL,NULL))
				{
				SetDlgItemText(hDlg,ID_TIFFDATE,(LPSTR)GlobalLock(hStr));
				GlobalUnlock(hStr);
				GlobalFree(hStr);
				}

			if(GetTiffImageInfoBuffer(hChain,nImage,STRIPOFFSETS,(HANDLE FAR*)&hStr,NULL,(DWORD FAR*)&dwValue))
				{
				SetDlgItemText(hDlg,ID_TIFFSTRIPS,ltoa(dwValue,Buff,10));
				GlobalFree(hStr);
				}

			if(GetTiffImageInfo(hChain,nImage,IMAGEWIDTH,(DWORD FAR*)&dwValue))
				SetDlgItemText(hDlg,ID_TIFFWIDTH,ltoa(dwValue,Buff,10));

			if(GetTiffImageInfo(hChain,nImage,IMAGELENGTH,(DWORD FAR*)&dwValue))
				SetDlgItemText(hDlg,ID_TIFFHEIGHT,ltoa(dwValue,Buff,10));

			dwValue = 1;
			if(GetTiffImageInfo(hChain,nImage,SAMPLESPERPIXEL,(DWORD FAR*)&dwValue))
				SetDlgItemText(hDlg,ID_TIFFPLANES,ltoa(dwValue,Buff,10));
			dwBits = dwValue;

			dwValue = 1;
			if(GetTiffImageInfo(hChain,nImage,BITSPERSAMPLE,(DWORD FAR*)&dwValue))
				SetDlgItemText(hDlg,ID_TIFFBITS,ltoa(dwValue,Buff,10));
			dwBits *= dwValue;

			dwValue = PI_GREYSCALE;
			GetTiffImageInfo(hChain,nImage,PHOTOMETRICINTERPRETATION,(DWORD FAR*)&dwValue);
			switch(dwValue){
				case PI_RGBCOLOR: 
					SetDlgItemText(hDlg,ID_TIFFCOLOR,(LPSTR)"RGB color");
					break;
				case PI_TRANSPMASK: 
					SetDlgItemText(hDlg,ID_TIFFCOLOR,(LPSTR)"Transp. mask");
					break;
				case PI_PALETTECOLOR: 
					SetDlgItemText(hDlg,ID_TIFFCOLOR,(LPSTR)"Palette color");
					break;
				case PI_GREYSCALE: 
					SetDlgItemText(hDlg,ID_TIFFCOLOR,(dwBits==1) ? (LPSTR)"Monochrome" : (LPSTR)"Greyscale");
					break;
				case PI_INV_GREYSCALE: 
					SetDlgItemText(hDlg,ID_TIFFCOLOR,(dwBits==1) ? (LPSTR)"Inv. monochrome" : (LPSTR)"Inv. greyscale");
					break;
			}

			dwValue = NOCOMPRESSION;
			if(GetTiffImageInfo(hChain,nImage,COMPRESSION,(DWORD FAR*)&dwValue))
				{
				switch(dwValue){
					case TIFF_CCITT: 
						SetDlgItemText(hDlg,ID_TIFFCOMPR,(LPSTR)"CCITT G3 No EOL");
						break;
					case CCITT_GROUP3: 
						SetDlgItemText(hDlg,ID_TIFFCOMPR,(LPSTR)"CCITT G3");
						dwValue = 0;
					 	wPredTag = GROUP3OPTIONS;
						break;
					case CCITT_GROUP4: 
						SetDlgItemText(hDlg,ID_TIFFCOMPR,(LPSTR)"CCITT G4");
						dwValue = 0;
					 	wPredTag = GROUP4OPTIONS;
						break;
					case LZW_COMPRESSION: 
						SetDlgItemText(hDlg,ID_TIFFCOMPR,(LPSTR)"LZW");
						dwValue = 1;
					 	wPredTag = PREDICTOR;
						break;
					case PACK_BITS: 
						SetDlgItemText(hDlg,ID_TIFFCOMPR,(LPSTR)"PackBits");
						break;
					}
				}

			if(wPredTag != 0)
				{
				GetTiffImageInfo(hChain,nImage,wPredTag,(DWORD FAR*)&dwValue);
				switch(wPredTag){
					case GROUP3OPTIONS: 
						SetDlgItemText(hDlg,ID_TIFFPRED,
								(dwValue & 1) ? (LPSTR)"2 dim." : (LPSTR)"1 dim.");
						break;
					case PREDICTOR: 
						SetDlgItemText(hDlg,ID_TIFFPRED,
								(dwValue == 1) ? (LPSTR)"Normal" : (LPSTR)"Differ.");
						break;
					}
				}

			SetFocus(GetDlgItem(hDlg,IDOK));

			return(FALSE);

		case WM_COMMAND:
			switch(wParam){
				case IDOK:
				case IDCANCEL:
					EndDialog(hDlg,(wParam==IDOK));
					break;

				default: 
					return(FALSE);
			} 

		default: 
			return(FALSE);
	}
	return(TRUE);
}



/**********************************************************************************
  Function
		AskDimsDlg

  Parameters
		hDlg						Dialog window handle.
		wMessage					Message sent.
		wParam					Additional info.
		lParam					Additional info.

  Returns
		FALSE if message is processed, TRUE otherwise.

  Description
		Dialog box function to enter image dimensions.

  Notes
		None.

  Author
		Hrasko_Gabor,	01/02/91		Initial version.
**********************************************************************************/

BOOL FAR PASCAL AskDimsDlg(HWND hDlg, WORD wMessage, WORD wParam, LONG lParam)
{
	char sWidth[10], sHeight[10];	
	WORD wWidth, wHeight;

	switch(wMessage){

		case WM_INITDIALOG: 
			SetFocus(GetDlgItem(hDlg,ID_IMGWIDTH));
			return(FALSE);

		case WM_COMMAND:

			switch(wParam){

				case IDOK:
					GetDlgItemText(hDlg,ID_IMGWIDTH,(LPSTR)sWidth,9);
					sWidth[9] = '\0';
					GetDlgItemText(hDlg,ID_IMGHEIGHT,(LPSTR)sHeight,9);
					sHeight[9] = '\0';
					wWidth = (WORD) atoi(sWidth);
					wHeight = (WORD) atoi(sHeight);
					if(wWidth > 0)
						if(wHeight > 0 || sHeight[0] == '0' || sHeight[0] == '\0')
							{
							BmInfo.bmiHeader.biWidth = wWidth;
							BmInfo.bmiHeader.biHeight = (wHeight) ? wHeight : UNDEF_HEIGHT;
							EndDialog(hDlg,TRUE);
							break; 
							}
					// Invalid values.
					MessageBeep(0);
					break;

				case IDCANCEL: 
					EndDialog(hDlg,FALSE);
					break;

				case ID_WIDTH1:
					SetDlgItemText(hDlg,ID_IMGWIDTH,(LPSTR)"1728");
					SetFocus(GetDlgItem(hDlg,ID_IMGWIDTH));
					break;

				case ID_WIDTH2:
					SetDlgItemText(hDlg,ID_IMGWIDTH,(LPSTR)"2048");
					SetFocus(GetDlgItem(hDlg,ID_IMGWIDTH));
					break;

				case ID_WIDTH3:
					SetDlgItemText(hDlg,ID_IMGWIDTH,(LPSTR)"2560");
					SetFocus(GetDlgItem(hDlg,ID_IMGWIDTH));
					break;

				default: 
					return(FALSE);

			} 

		default: 
			return(FALSE);

	}

	return(TRUE);
}



/**********************************************************************************
  Function
		TiffDlg

  Parameters
		hDlg						Dialog window handle.
		wMessage			  Message sent.
		wParam					Additional info.
		lParam					Additional info.

  Returns
		FALSE if message is processed, TRUE otherwise.

  Description
		Dialog box function to select compression type and enter image
		description tag info. CCITT methods are not available if the image
		is not monochrome.

  Notes
		None.

  Author
		Hrasko_Gabor,	01/02/91		Initial version.
**********************************************************************************/

BOOL FAR PASCAL TiffDlg(HWND hDlg, WORD wMessage, WORD wParam, LONG lParam)
{
	static int nBmColors;
	BITMAP Bm;

	switch(wMessage){

		case WM_INITDIALOG: 
			{
			char lpBuff[200];

 			sprintf(lpBuff,"Saving %s",Title);
			SetWindowText(hDlg,(LPSTR)lpBuff);

			wCompMode = ID_NOCOMP;
			CheckRadioButton(hDlg,ID_NOCOMP,ID_CCITTG4,wCompMode);

			GetObject(LOWORD(lParam),sizeof(BITMAP),(LPSTR)&Bm);

			// Sets global variables.
			nClrsImprt = nBmColors = 1 << (Bm.bmPlanes*Bm.bmBitsPixel);
			nPhotomInterp = (nClrsImprt == 2) ? 1 : 3;

			if (nClrsImprt == 2)
				{
				// Differencing has no sense in case of monochrome images.
				EnableWindow(GetDlgItem(hDlg,ID_LZW_DIFF),FALSE);
				// Can't save as color or greyscale if it is monochrome.
				EnableWindow(GetDlgItem(hDlg,ID_GREY),FALSE);
				EnableWindow(GetDlgItem(hDlg,ID_COLOR),FALSE);
				CheckRadioButton(hDlg,ID_MONO,ID_COLOR,ID_MONO);
				}
			else
				{
				// CCITT compressions work only on monochrome images.
				EnableWindow(GetDlgItem(hDlg,ID_CCITTG2),FALSE);
				EnableWindow(GetDlgItem(hDlg,ID_CCITTG31D),FALSE);
				EnableWindow(GetDlgItem(hDlg,ID_CCITTG32D),FALSE);
				EnableWindow(GetDlgItem(hDlg,ID_CCITTG4),FALSE);
				CheckRadioButton(hDlg,ID_MONO,ID_COLOR,ID_COLOR);
				}
 
			SetFocus(GetDlgItem(hDlg,IDOK));

			return(FALSE);
			}

		case WM_COMMAND:

			switch(wParam){

				case IDOK:
					GetDlgItemText(hDlg,ID_TITLE,Title,100);
					Title[99] = '\0';
					GetDlgItemText(hDlg,ID_ARTIST,Artist,100);
					Artist[99] = '\0';
					EndDialog(hDlg,TRUE);
					break; 

				case IDCANCEL: 
					EndDialog(hDlg,FALSE);
					break;

				case ID_NOCOMP: 
				case ID_PACKBITS: 
				case ID_LZW: 
				case ID_LZW_DIFF: 
				case ID_CCITTG2: 
				case ID_CCITTG31D: 
				case ID_CCITTG32D: 
				case ID_CCITTG4: 
					wCompMode = wParam;
					CheckRadioButton(hDlg,ID_NOCOMP,ID_CCITTG4,wParam);
					break;
	
				case ID_MONO:
					nClrsImprt = 2;
					nPhotomInterp = 1;
					CheckRadioButton(hDlg,ID_MONO,ID_COLOR,wParam);
					EnableWindow(GetDlgItem(hDlg,ID_LZW_DIFF),FALSE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG2),TRUE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG31D),TRUE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG32D),TRUE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG4),TRUE);
					if(wCompMode == ID_LZW_DIFF)
						{
						wCompMode = ID_NOCOMP;
						CheckRadioButton(hDlg,ID_NOCOMP,ID_CCITTG4,wCompMode);
						}
					break;

				case ID_GREY:
					nClrsImprt = nBmColors;
					nPhotomInterp = 1;
					CheckRadioButton(hDlg,ID_MONO,ID_COLOR,wParam);
					EnableWindow(GetDlgItem(hDlg,ID_LZW_DIFF),TRUE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG2),FALSE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG31D),FALSE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG32D),FALSE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG4),FALSE);
					if(wCompMode >= ID_CCITTG2)
						{
						wCompMode = ID_NOCOMP;
						CheckRadioButton(hDlg,ID_NOCOMP,ID_CCITTG4,wCompMode);
						}
					break;

				case ID_COLOR:
					nClrsImprt = nBmColors;
					nPhotomInterp = 3;
					CheckRadioButton(hDlg,ID_MONO,ID_COLOR,wParam);
					EnableWindow(GetDlgItem(hDlg,ID_LZW_DIFF),TRUE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG2),FALSE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG31D),FALSE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG32D),FALSE);
					EnableWindow(GetDlgItem(hDlg,ID_CCITTG4),FALSE);
					if(wCompMode >= ID_CCITTG2)
						{
						wCompMode = ID_NOCOMP;
						CheckRadioButton(hDlg,ID_NOCOMP,ID_CCITTG4,wCompMode);
						}
					break;

				default: 
					return(FALSE);

			} 

		default: 
			return(FALSE);

	}

	return(TRUE);
}



//---------------------------- Module Code (Locals) ---------------------------

//--------------------------------- Footnotes ---------------------------------

//-------------------------------- End of file --------------------------------
